
/*      DEMO.C
 *
 * MIDAS megademo
 *
 * Copyright 1996 Petteri Kangaslampi
*/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <conio.h>
#include <math.h>


#define __WC32__

//#include "midas.h"
/*      midas.h
 *
 * Simplified MIDAS Sound System API
 *
 * Id: midas.h,v 1.15 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#ifndef __MIDAS_H
#define __MIDAS_H

/****************************************************************************\
*      NULL pointer definition (independence on library includes):
\****************************************************************************/

#ifndef NULL
    #define NULL 0L
#endif



/****************************************************************************\
*      MIDAS version defines
\****************************************************************************/

#define MVERSION 0.6.0
#define MVERSTR "0.6.0"
#define MVERNUM 0x0600



/****************************************************************************\
*      MIDAS header files:
\****************************************************************************/

//#include "lang.h"
/*      LANG.H
 *
 * Destination language dependent macros and conditional compilation
 *
 * $Id: lang.h,v 1.9 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#ifndef __LANG_H
#define __LANG_H


#ifdef __WC32__
    #define CALLING __cdecl
    #define GLOBALVAR
    #define __C__
    #define __PROTMODE__
    #define __32__
    #define NOEMS
    #define __DPMI__
    #define __FLATMODE__
    #define EMPTYARRAY

    #ifdef __cplusplus
        /* Throw a couple of Watcom C++ watnings out of our way to level 9
           - I have no idea how to work around these in the code while
           maintaining Visual C compatibility */
        #pragma warning 604 9
        #pragma warning 594 9
    #endif
#else
#ifdef __VC32__
    #define CALLING cdecl
    #define GLOBALVAR
    #define __C__
    #define __PROTMODE__
    #define __32__
    #define NOEMS
    #define __DPMI__
    #define __FLATMODE__
    #define __WIN32__

    /* Disable warning about 0-sized arrays in structures: */
    #pragma warning(disable:4200)

    #define EMPTYARRAY
#else
#ifdef __LINUX__
    #define CALLING
    #define GLOBALVAR
    #define __C__
    #define __PROTMODE__
    #define __32__
    #define NOEMS
    #define __FLATMODE__
    #define NOTIMER
    #define SUPPORTSTREAMS

    #define EMPTYARRAY 0
#else
    #error NO COMPILER DEFINED!
#endif
#endif
#endif

#ifdef __16__
    typedef unsigned char U8;
    typedef signed char S8;
    typedef unsigned short U16;
    typedef signed short S16;
    typedef unsigned long U32;
    typedef signed long S32;
    typedef unsigned int UINT;
    typedef signed int SINT;
#else
    typedef unsigned char U8;
    typedef signed char S8;
    typedef unsigned short U16;
    typedef signed short S16;
    typedef unsigned long U32;
    typedef signed long S32;
    typedef unsigned int UINT;
    typedef signed int SINT;
#endif

#if defined(__WINDOWS__) || defined(__NT__) || defined(__VC32__)
    #define SUPPORTSTREAMS
    #define __WIN32__
#endif

#define RCSID(x) x


#endif


/*
 * $Log: lang.h,v $
 * Revision 1.9  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.8  1996/09/28 08:12:40  jpaana
 * Enabled SUPPORTSTREAMS for Linux
 *
 * Revision 1.7  1996/09/01 19:03:52  pekangas
 * Removed a couple of warnings from Watcom C++ in C++ mode
 *
 * Revision 1.6  1996/07/16 20:21:00  pekangas
 * Added support for Visual C
 *
 * Revision 1.5  1996/07/13 17:29:08  pekangas
 * Removed cdecl calling convention redefinition pragma
 *
 * Revision 1.4  1996/05/26 16:44:22  pekangas
 * Defined SUPPORTSTREAMS and __WIN32__ when compiling for NT
 *
 * Revision 1.3  1996/05/24 16:58:22  pekangas
 * Added #define EMPTYARRAY - used to declare empty arrays at the end of a structure: type array[EMPTYARRAY] to gain portability.
 *
 * Revision 1.2  1996/05/24 16:20:36  jpaana
 * Added __LINUX__
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/


//#include "mtypes.h"
/*      MTYPES.H
 *
 * MIDAS Sound System common type definitions
 *
 * $Id: mtypes.h,v 1.4 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#ifndef __MTYPES_H
#define __MTYPES_H

typedef unsigned char uchar;
#ifndef _LINUX_TYPES_H
typedef unsigned short ushort;
typedef unsigned long ulong;
#endif

#ifndef NULL
#define NULL 0L
#endif

#endif


/*
 * $Log: mtypes.h,v $
 * Revision 1.4  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.3  1996/09/21 17:40:45  jpaana
 * Fixed ushort and ulong warnings on Linux
 *
 * Revision 1.2  1996/05/25 15:49:57  jpaana
 * Nothing really
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/



//#include "errors.h"
/*      errors.h
 *
 * MIDAS Sound System error codes and error message strings
 *
 * $Id: errors.h,v 1.6 1996/10/20 20:53:36 pekangas Exp $
 *
 * Copyright 1996 Petteri Kangaslampi and Jarno Paananen
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __errors_h
#define __errors_h


#define MAXERRORS 256       /* maximum number of errors to store */


/****************************************************************************\
*       enum ErrorCodes
*       ---------------
* Description:  MIDAS Sound System error codes
\****************************************************************************/

enum ErrorCodes
{
    OK = 0,                             /* no error */
    errUndefined,                       /* undefined error */
    errOutOfMemory,                     /* out of (conventional) memory */
    errHeapCorrupted,                   /* (conventional memory) heap
                                           corrupted */
    errInvalidBlock,                    /* invalid memory block */
    errOutOfEMS,                        /* out of EMS memory */
    errEMSHeapCorrupted,                /* EMS heap corrupted */
    errInvalidEMSBlock,                 /* invalid EMS memory block */
    errEMMFailure,                      /* Expanded Memory Manager failure */
    errOutOfCardMemory,                 /* out of soundcard memory */
    errCardHeapCorrupted,               /* soundcard heap corrupted */
    errInvalidCardBlock,                /* invalid soundcard memory block */
    errNoSampleHandles,                 /* out of sample handles */
    errFileOpen,                        /* unable to open file */
    errFileRead,                        /* unable to read file */
    errInvalidModule,                   /* invalid module file */
    errInvalidInst,                     /* invalid instrument in module */
    errInvalidPatt,                     /* invalid pattern data in module */
    errInvalidChanNumber,               /* invalid channel number */
    errInvalidSampleHandle,             /* invalid sample handle */
    errNoChannels,                      /* Sound Device channels not open */
    errSDFailure,                       /* Sound Device hardware failure */
    errInvalidArguments,                /* invalid function arguments */
    errFileNotFound,                    /* file does not exist */
    errInvalidFileHandle,               /* invalid file handle */
    errAccessDenied,                    /* access denied */
    errFileExists,                      /* file exists */
    errTooManyFiles,                    /* too many open files */
    errDiskFull,                        /* disk full */
    errEndOfFile,                       /* unexpected end of file */
    errInvalidPath,                     /* invalid path */
    errFileWrite,                       /* unable to write file */
    errVDSLock,                         /* unable to lock VDS DMA buffer */
    errVDSUsage,                        /* unable to use Virtual DMA */
    errBadVDS,                          /* invalid VDS version */
    errDPMIFailure,                     /* DPMI failure */
    errInvalidDescriptor,               /* invalid segment descriptor */
    errOutOfResources,                  /* out of system resources */
    errInvalidDevice,                   /* invalid device used */
    errUnsupported,                     /* unsupported function used */
    errDeviceNotAvailable,              /* device not available */
    errDeviceBusy,                      /* device is busy */
    errBadMode,                         /* unsupported output mode used */
    errUnableToLock,                    /* unable to lock memory */
    errNumErrorCodes
};



/****************************************************************************\
*       enum FunctionIDs
*       ----------------
* Description:  ID numbers for first functions in all modules
\****************************************************************************/

enum FunctionIDs
{
    ID_error = 0,                       /* error handling */
    ID_dma = 100,                       /* DMA handling routines */
    ID_dsm = 200,                       /* Digital Sound Mixer */
    ID_ems = 300,                       /* EMS heap manager */
    ID_mem = 400,                       /* Conventional memory management */
    ID_mod = 500,                       /* Protracker Module Player */
    ID_s3m = 600,                       /* Scream Tracker 3 Module Player */
    ID_mtm = 700,                       /* Multitracker Module Player */
    ID_tmr = 1000,                      /* TempoTimer */
    ID_vu = 1100,                       /* Real VU meters */
    ID_rf = 1200,                       /* Raw file I/O */
    ID_file = 1300,                     /* High-level file I/O */
    ID_dpmi = 1400,                     /* DPMI functions */
    ID_gmp = 1500,                      /* Generic Module Player */
    ID_fx = 1600,                       /* Sound effects library */
    ID_str = 1700,                      /* Stream library */
    ID_gus = 2000,                      /* GUS Sound Device */
    ID_pas = 2100,                      /* PAS Sound Device */
    ID_wss = 2200,                      /* WSS Sound Device */
    ID_sb = 2300,                       /* SB Sound Device */
    ID_winw = 2400,                     /* Windows Wave Sound Device */
    ID_oss = 2500,                      /* OSS Sound Device */
    ID_gdc = 2600,                      /* GUS Mixing Sound Device */
    ID_awe = 2700,                      /* AWE32 Sound Device */
    ID_nsnd = 2900,                     /* No Sound Sound Device */
    ID_mixsd = 3000                     /* Common mixing Sound Device
                                           functions */
};



#ifdef __cplusplus
extern "C" {
#endif


#ifdef DEBUG

/****************************************************************************\
*       struct errRecord
*       ----------------
* Description:  Error record for error list
\****************************************************************************/

typedef struct
{
    int         errorCode;              /* error code number */
    unsigned    functID;                /* ID for function that caused the
                                           error */
} errRecord;


extern errRecord GLOBALVAR  errorList[MAXERRORS];       /* error list */
extern unsigned GLOBALVAR   numErrors;  /* number of errors in list */



/****************************************************************************\
*
* Function:     void errAdd(int errorCode, unsigned functID);
*
* Description:  Add an error to error list
*
* Input:        int errorCode           error code
*               unsigned functID        ID for function that caused the error
*
\****************************************************************************/

void CALLING errAdd(int errorCode, unsigned functID);




/****************************************************************************\
*
* Function:     void errClearList(void)
*
* Description:  Clears the error list. Can be called if a error has been
*               handled without exiting the program to avoid filling the
*               error list with handled errors.
*
\****************************************************************************/

void CALLING errClearList(void);




/****************************************************************************\
*
* Function:     void errPrintList(void);
*
* Description:  Prints the error list to stderr
*
\****************************************************************************/

void CALLING errPrintList(void);


#endif




/****************************************************************************\
*
* Function:     void errErrorExit(char *msg)
*
* Description:  Set up standard text mode, print an error message and exit
*
* Input:        char *msg               pointer to error message, ASCIIZ
*
\****************************************************************************/

void CALLING errErrorExit(char *msg);






/* error message string pointers: */
extern char * GLOBALVAR errorMsg[errNumErrorCodes];


#ifdef __cplusplus
}
#endif




//win-
#ifdef ERROR
#undef ERROR
#endif


/* ERROR - adds an error to the MIDAS error list if DEBUG is defined.
   Does nothing otherwise. */

#ifdef DEBUG
    #define ERROR(errCode, functID) errAdd(errCode, functID)
#else
    #define ERROR(errCode, functID)
#endif


/* PASSERROR - passes error value in variable error on */

#ifdef DEBUG
    #define PASSERROR(functID) { errAdd(error, functID); return error; }
#else
    #define PASSERROR(functID) return error;
#endif



/* CLEARERRORLIST - clear the MIDAS error list if DEBUG is defined. */
#ifdef DEBUG
    #define CLEARERRORLIST errClearList();
#else
    #define CLEARERRORLIST ;
#endif



#endif  /* #ifndef __errors_h */


/*
 * $Log: errors.h,v $
 * Revision 1.6  1996/10/20 20:53:36  pekangas
 * Added AWE32 Sound Device ID
 *
 * Revision 1.5  1996/09/22 23:15:33  pekangas
 * Added ID_fx and ID_str
 *
 * Revision 1.4  1996/09/21 16:38:00  jpaana
 * Changed uss to oss
 *
 * Revision 1.3  1996/06/05 19:40:35  jpaana
 * Changed usswave to uss
 *
 * Revision 1.2  1996/05/24 20:40:12  jpaana
 * Added USSWave
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/





//#include "mglobals.h"
/*      MGLOBALS.H
 *
 * MIDAS Sound System global variables
 *
 * $Id: mglobals.h,v 1.4 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#ifndef __MGLOBALS_H
#define __MGLOBALS_H


#ifdef __cplusplus
extern "C" {
#endif


extern int GLOBALVAR mUseEMS;           /* 1 if EMS should be used */
extern int GLOBALVAR mEnableSurround;   /* 1 if surround sound is enabled */
extern int GLOBALVAR mBufferLength;     /* buffer length in milliseconds */
extern int GLOBALVAR mBufferBlocks;     /* number of buffer blocks
                                           (not applicable to all SDs) */
extern int GLOBALVAR mDefaultFramerate; /* default framerate (in 100*Hz), used
                                           when screen sync is not available*/
extern int GLOBALVAR mSyncScreen;       /* 1 if timer can be synchronized to
                                           screen */




#ifdef __cplusplus
}
#endif




#endif


/*
 * $Log: mglobals.h,v $
 * Revision 1.4  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.3  1996/05/31 21:40:35  pekangas
 * Added mSyncScreen and mDefaultFramerate
 *
 * Revision 1.2  1996/05/25 09:32:26  pekangas
 * Added mBufferLength and mBufferBlocks
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/




//#include "mmem.h"
/*      MMEM.H
 *
 * MIDAS Sound System memory handling routines
 *
 * $Id: mmem.h,v 1.2 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __MEM_H
#define __MEM_H


#ifdef __cplusplus
extern "C" {
#endif



/****************************************************************************\
*
* Function:     int memAlloc(unsigned len, void **blk);
*
* Description:  Allocates a block of conventional memory
*
* Input:        unsigned len            Memory block length in bytes
*               void **blk              Pointer to memory block pointer
*
* Returns:      MIDAS error code.
*               Pointer to allocated block stored in *blk, NULL if error.
*
\****************************************************************************/

int CALLING memAlloc(unsigned len, void **blk);



/****************************************************************************\
*
* Function:     int memFree(void *blk);
*
* Description:  Deallocates a memory block allocated with memAlloc()
*
* Input:        void *blk               Memory block pointer
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int CALLING memFree(void *blk);




/****************************************************************************\
*       enum memFunctIDs
*       ----------------
* Description:  ID numbers for memory handling functions
\****************************************************************************/

enum memFunctIDs
{
    ID_memAlloc = ID_mem,
    ID_memFree
};




#ifdef __cplusplus
}
#endif

#endif


/*
 * $Log: mmem.h,v $
 * Revision 1.2  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/



//#include "file.h"
/*      FILE.H
 *
 * High-level file I/O for MIDAS Sound System
 *
 * $Id: file.h,v 1.2 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#ifndef __FILE_H
#define __FILE_H

//#include "rawfile.h"
/*      RAWFILE.H
 *
 * Raw file I/O for MIDAS Sound System
 *
 * $Id: rawfile.h,v 1.3 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __RAWFILE_H
#define __RAWFILE_H



#ifdef __WIN32__
    #define rfFile_fileHandle U32 f;
#else
    #ifndef FILE
    #include <stdio.h>
    #endif
    #define rfFile_fileHandle FILE *f;
#endif


/****************************************************************************\
*       struct rfFile
*       -------------
* Description:  File state structure
\****************************************************************************/

typedef struct
{
    rfFile_fileHandle
} rfFile;




/****************************************************************************\
*       typedef rfHandle;
*       -----------------
* Description: Raw file I/O file handle
\****************************************************************************/

typedef rfFile* rfHandle;



/****************************************************************************\
*       enum rfOpenMode
*       ---------------
* Description:  File opening mode. Used by rfOpen()
\****************************************************************************/

enum rfOpenMode
{
    rfOpenRead = 1,                     /* open file for reading */
    rfOpenWrite = 2,                    /* open file for writing */
    rfOpenReadWrite = 3                 /* open file for both reading and
                                           writing */
};



/****************************************************************************\
*       enum rfSeekMode
*       ---------------
* Description:  File seeking mode. Used by rfSeek()
\****************************************************************************/

enum rfSeekMode
{
    rfSeekAbsolute = 1,                 /* seek to an absolute position from
                                           the beginning of the file */
    rfSeekRelative = 2,                 /* seek to a position relative to
                                           current position */
    rfSeekEnd = 3                       /* relative to the end of file */
};



#ifdef __cplusplus
extern "C" {
#endif


/****************************************************************************\
*
* Function:     int rfOpen(char *fileName, int openMode, rfHandle *file);
*
* Description:  Opens a file for reading or writing
*
* Input:        char *fileName          name of file
*               int openMode            file opening mode, see enum rfOpenMode
*               rfHandle *file          pointer to file handle
*
* Returns:      MIDAS error code.
*               File handle is stored in *file.
*
\****************************************************************************/

int CALLING rfOpen(char *fileName, int openMode, rfHandle *file);




/****************************************************************************\
*
* Function:     int rfClose(rfHandle file);
*
* Description:  Closes a file opened with rfOpen().
*
* Input:        rfHandle file           handle of an open file
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING rfClose(rfHandle file);




/****************************************************************************\
*
* Function:     int rfGetSize(rfHandle file, long *fileSize);
*
* Description:  Get the size of a file
*
* Input:        rfHandle file           handle of an open file
*               ulong *fileSize         pointer to file size
*
* Returns:      MIDAS error code.
*               File size is stored in *fileSize.
*
\****************************************************************************/

int CALLING rfGetSize(rfHandle file, long *fileSize);




/****************************************************************************\
*
* Function:     int rfRead(rfHandle file, void *buffer, ulong numBytes);
*
* Description:  Reads binary data from a file
*
* Input:        rfHandle file           file handle
*               void *buffer            reading buffer
*               ulong numBytes          number of bytes to read
*
* Returns:      MIDAS error code.
*               Read data is stored in *buffer, which must be large enough
*               for it.
*
\****************************************************************************/

int CALLING rfRead(rfHandle file, void *buffer, ulong numBytes);




/****************************************************************************\
*
* Function:     int rfWrite(rfHandle file, void *buffer, ulong numBytes);
*
* Description:  Writes binary data to a file
*
* Input:        rfHandle file           file handle
*               void *buffer            pointer to data to be written
*               ulong numBytes          number of bytes to write
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING rfWrite(rfHandle file, void *buffer, ulong numBytes);




/****************************************************************************\
*
* Function:     int rfSeek(rfHandle file, long newPosition, int seekMode);
*
* Description:  Seeks to a new position in file. Subsequent reads and writes
*               go to the new position.
*
* Input:        rfHandle file           file handle
*               long newPosition        new file position
*               int seekMode            file seek mode, see enum rfSeekMode
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING rfSeek(rfHandle file, long newPosition, int seekMode);




/****************************************************************************\
*
* Function:     int rfGetPosition(rfHandle file, long *position);
*
* Description:  Reads the current position in a file
*
* Input:        rfHandle file           file handle
*               long *position          pointer to file position
*
* Returns:      MIDAS error code.
*               Current file position is stored in *position.
*
\****************************************************************************/

int CALLING rfGetPosition(rfHandle file, long *position);




/****************************************************************************\
*
* Function:     int rfFileExists(char *fileName, int *exists);
*
* Description:  Checks if a file exists or not
*
* Input:        char *fileName          file name, ASCIIZ
*               int *exists             pointer to file exists status
*
* Returns:      MIDAS error code.
*               *exists contains 1 if file exists, 0 if not.
*
\****************************************************************************/

int CALLING rfFileExists(char *fileName, int *exists);



#ifdef __cplusplus
}
#endif



/****************************************************************************\
*       enum rfFunctIDs
*       ---------------
* Description:  ID numbers for raw file I/O functions
\****************************************************************************/

enum rfFunctIDs
{
    ID_rfOpen = ID_rf,
    ID_rfClose,
    ID_rfGetSize,
    ID_rfRead,
    ID_rfWrite,
    ID_rfSeek,
    ID_rfGetPosition,
    ID_rfFileExists
};


#endif


/*
 * $Log: rawfile.h,v $
 * Revision 1.3  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.2  1996/08/13 20:48:13  pekangas
 * Changed for Win32 raw file I/O (rawfile.c)
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/


/****************************************************************************\
*       struct fileFile
*       ---------------
* Description:  File state structure
\****************************************************************************/

typedef struct
{
    rfHandle    rf;
} fileFile;




/****************************************************************************\
*       typedef fileHandle
*       ------------------
* Description: High-level file I/O file handle
\****************************************************************************/

typedef fileFile* fileHandle;



/****************************************************************************\
*       enum fileOpenMode
*       -----------------
* Description:  File opening mode. Used by fileOpen()
\****************************************************************************/

enum fileOpenMode
{
    fileOpenRead = 1,                   /* open file for reading */
    fileOpenWrite = 2,                  /* open file for writing */
    fileOpenReadWrite = 3               /* open file for both reading and
                                           writing */
};



/****************************************************************************\
*       enum fileSeekMode
*       -----------------
* Description:  File seeking mode. Used by fileSeek()
\****************************************************************************/

enum fileSeekMode
{
    fileSeekAbsolute = 1,               /* seek to an absolute position from
                                           the beginning of the file */
    fileSeekRelative = 2,               /* seek to a position relative to
                                           current position */
    fileSeekEnd = 3                     /* seek relative to the end of file */
};



#ifdef __cplusplus
extern "C" {
#endif


/****************************************************************************\
*
* Function:     int fileOpen(char *fileName, int openMode, fileHandle *file);
*
* Description:  Opens a file for reading or writing
*
* Input:        char *fileName          name of file
*               int openMode            file opening mode, see enum rfOpenMode
*               fileHandle *file        pointer to file handle
*
* Returns:      MIDAS error code.
*               File handle is stored in *file.
*
\****************************************************************************/

int CALLING fileOpen(char *fileName, int openMode, fileHandle *file);




/****************************************************************************\
*
* Function:     int fileClose(fileHandle file);
*
* Description:  Closes a file opened with fileOpen().
*
* Input:        fileHandle file         handle of an open file
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fileClose(fileHandle file);




/****************************************************************************\
*
* Function:     int fileGetSize(fileHandle file, long *fileSize);
*
* Description:  Get the size of a file
*
* Input:        fileHandle file         handle of an open file
*               ulong *fileSize         pointer to file size
*
* Returns:      MIDAS error code.
*               File size is stored in *fileSize.
*
\****************************************************************************/

int CALLING fileGetSize(fileHandle file, long *fileSize);




/****************************************************************************\
*
* Function:     int fileRead(fileHandle file, void *buffer, ulong numBytes);
*
* Description:  Reads binary data from a file
*
* Input:        fileHandle file         file handle
*               void *buffer            reading buffer
*               ulong numBytes          number of bytes to read
*
* Returns:      MIDAS error code.
*               Read data is stored in *buffer, which must be large enough
*               for it.
*
\****************************************************************************/

int CALLING fileRead(fileHandle file, void *buffer, ulong numBytes);




/****************************************************************************\
*
* Function:     int fileWrite(fileHandle file, void *buffer, ulong numBytes);
*
* Description:  Writes binary data to a file
*
* Input:        fileHandle file         file handle
*               void *buffer            pointer to data to be written
*               ulong numBytes          number of bytes to write
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fileWrite(fileHandle file, void *buffer, ulong numBytes);




/****************************************************************************\
*
* Function:     int fileSeek(fileHandle file, long newPosition, int seekMode);
*
* Description:  Seeks to a new position in file. Subsequent reads and writes
*               go to the new position.
*
* Input:        fileHandle file         file handle
*               long newPosition        new file position
*               int seekMode            file seek mode, see enum rfSeekMode
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fileSeek(fileHandle file, long newPosition, int seekMode);




/****************************************************************************\
*
* Function:     int fileGetPosition(fileHandle file, long *position);
*
* Description:  Reads the current position in a file
*
* Input:        fileHandle file         file handle
*               long *position          pointer to file position
*
* Returns:      MIDAS error code.
*               Current file position is stored in *position.
*
\****************************************************************************/

int CALLING fileGetPosition(fileHandle file, long *position);




/****************************************************************************\
*
* Function:     int fileExists(char *fileName, int *exists);
*
* Description:  Checks if a file exists or not
*
* Input:        char *fileName          file name, ASCIIZ
*               int *exists             pointer to file exists status
*
* Returns:      MIDAS error code.
*               *exists contains 1 if file exists, 0 if not.
*
\****************************************************************************/

int CALLING fileExists(char *fileName, int *exists);




#ifdef __cplusplus
}
#endif



/****************************************************************************\
*       enum fileFunctIDs
*       -----------------
* Description:  ID numbers for high-level file I/O functions
\****************************************************************************/

enum fileFunctIDs
{
    ID_fileOpen = ID_file,
    ID_fileClose,
    ID_fileGetSize,
    ID_fileRead,
    ID_fileWrite,
    ID_fileSeek,
    ID_fileGetPosition,
    ID_fileExists
};

#endif

/*
 * $Log: file.h,v $
 * Revision 1.2  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/



//#include "sdevice.h"
/*      SDEVICE.H
 *
 * Sound Device definitions
 *
 * $Id: sdevice.h,v 1.9 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __SDEVICE_H
#define __SDEVICE_H

#ifdef __16__
#define SMPMAX 65519                    /* max sample length (65536-16 - 1) */
#else
#define SMPMAX 2147483631               /* max sample length (2^31 - 17) */
#endif
#define MAXSAMPLES 256                  /* maximum number of samples */

#define DMABUFLEN 25                    /* mixing buffer length is a 1/25th of
                                           a second */



/****************************************************************************\
*       enum sdSampleType
*       -----------------
* Description:  Sample types
\****************************************************************************/

enum sdSampleType
{
    smpNone = 0,                        /* no sample */
    smp8bitMono = 1,                    /* 8-bit mono unsigned sample */
    smp8bit = smp8bitMono,              /* for compatibility */
    smp16bitMono = 2,                   /* 16-bit mono signed sample */
    smp16bit = smp16bitMono,            /* for compatibility */
    smp8bitStereo = 3,                  /* 8-bit stereo unsigned sample */
    smp16bitStereo = 4                  /* 16-bit stereo signed sample */
};




/****************************************************************************\
*       enum sdLoopMode
*       ---------------
* Description:  Sample looping modes
\****************************************************************************/

enum sdLoopMode
{
    sdLoopNone = 0,                     /* no looping */
    sdLoop1 = 1,                        /* 1 loop, release ends note */
    sdLoop1Rel = 2,                     /* 1 loop, sample data after loop is
                                           played when note is released */
    sdLoop2 = 3,                        /* 2 loops, when note is released
                                           playing is continued looping the
                                           second loop */
    sdLoopAmigaNone = 4,                /* Amiga compatible looping, no
                                           loop - ALE logic enabled anyway */
    sdLoopAmiga = 5                     /* Amiga compatible looping - 1
                                           loop, ALE logic enabled */
};




/****************************************************************************\
*       enum sdLoopType
*       ---------------
* Description:  Sample looping type for one loop
\****************************************************************************/

enum sdLoopType
{
    loopNone = 0,                       /* no looping */
    loopUnidir = 1,                     /* unidirectional loop */
    loopBidir = 2                       /* bidirectional loop */
};




/****************************************************************************\
*       enum sdSmpPos
*       -------------
* Description:  Sample positions in memory
\****************************************************************************/

enum sdSmpPos
{
    sdSmpNone = 0,                   /* no sample */
    sdSmpConv,                       /* conventional memory */
    sdSmpEMS                         /* EMS */
};




/****************************************************************************\
*       struct sdSample
*       ---------------
* Description:  Sound Device sample information structure
\****************************************************************************/

typedef struct
{
    uchar       *sample;                /* sample data pointer */
    int         samplePos;              /* sample position in memory, see enum
                                           sdSmpPos */
    int         sampleType;             /* sample type, see enum
                                           sdSampleType */
    unsigned    sampleLength;           /* sample length */
    int         loopMode;               /* sample looping mode, see enum
                                           sdLoopMode */
    unsigned    loop1Start;             /* first loop start */
    unsigned    loop1End;               /* first loop end */
    int         loop1Type;              /* first loop type, see enum
                                           sdLoopType */
    unsigned    loop2Start;             /* second loop start */
    unsigned    loop2End;               /* second loop end */
    int         loop2Type;              /* second loop type, see enum
                                           sdLoopType */
} sdSample;




/****************************************************************************\
*       enum sdPanning
*       --------------
* Description:  Sound Device panning values. Legal values range from
*               panLeft to panRight, in steps of 1, plus panSurround.
*               Surround sound is played from middle if surround is not
*               enabled.
\****************************************************************************/

enum sdPanning
{
    panLeft = -64,                      /* left speaker */
    panMiddle = 0,                      /* middle (both speakers) */
    panRight = 64,                      /* right speaker */
    panSurround = 0x80                  /* surround sound */
};




/****************************************************************************\
*       enum sdMode
*       -----------
* Description:  Possible SoundDevice output modes
\****************************************************************************/

enum sdMode
{
    sdMono = 1,                         /* mono */
    sdStereo = 2,                       /* stereo */

    sd8bit = 4,                         /* 8-bit output */
    sd16bit = 8                         /* 16-bit output */
};




/****************************************************************************\
*       enum sdConfigBits
*       -----------------
* Description:  Sound Device configuration information bits
\****************************************************************************/

enum sdConfigBits
{
    sdUsePort = 1,                      /* SD uses an I/O port */
    sdUseIRQ = 2,                       /* SD uses an IRQ */
    sdUseDMA = 4,                       /* SD uses a DMA channel */
    sdUseMixRate = 8,                   /* SD uses the set mixing rate */
    sdUseOutputMode = 16,               /* SD uses the set output mode */
    sdUseDSM = 32                       /* SD uses software mixing (DSM) */
};




/****************************************************************************\
*       struct SoundDevice
*       ------------------
* Description:  SoundDevice structure. See SDEVICE.TXT for documentation
\****************************************************************************/

typedef struct
{
    unsigned    tempoPoll;
    unsigned    configBits;             /* Configuration info bits. See enum
                                           sdConfigBits. */
    unsigned    port;                   /* Sound Device I/O port address */
    unsigned    IRQ;                    /* Sound Device IRQ number */
    unsigned    DMA;                    /* Sound Device DMA channel number */
    unsigned    cardType;               /* Sound Device sound card type.
                                           Starting from 1, 0 means
                                           autodetect */
    unsigned    numCardTypes;           /* number of different sound card
                                           types for this Sound Device */
    unsigned    modes;                  /* Possible modes for this SD,
                                           see enum sdMode. Updated by
                                           Detect() */

    char        *name;                  /* pointer to Sound Device name */
    char        **cardNames;            /* pointer to an array of pointers to
                                           sound card name strings */
    unsigned    numPortAddresses;       /* number of possible port
                                           addresses in table */
    unsigned    *portAddresses;         /* pointer to an array of possible
                                           I/O port addresses */


    int (CALLING *Detect)(int *result);
    int (CALLING *Init)(unsigned mixRate, unsigned mode);
    int (CALLING *Close)(void);
    int (CALLING *GetMixRate)(unsigned *mixRate);
    int (CALLING *GetMode)(unsigned *mode);
    int (CALLING *OpenChannels)(unsigned channels);
    int (CALLING *CloseChannels)(void);
    int (CALLING *ClearChannels)(void);
    int (CALLING *Mute)(int mute);
    int (CALLING *Pause)(int pause);
    int (CALLING *SetMasterVolume)(unsigned masterVolume);
    int (CALLING *GetMasterVolume)(unsigned *masterVolume);
    int (CALLING *SetAmplification)(unsigned amplification);
    int (CALLING *GetAmplification)(unsigned *amplification);
    int (CALLING *PlaySound)(unsigned channel, ulong rate);
    int (CALLING *ReleaseSound)(unsigned channel);
    int (CALLING *StopSound)(unsigned channel);
    int (CALLING *SetRate)(unsigned channel, ulong rate);
    int (CALLING *GetRate)(unsigned channel, ulong *rate);
    int (CALLING *SetVolume)(unsigned channel, unsigned volume);
    int (CALLING *GetVolume)(unsigned channel, unsigned *volume);
    int (CALLING *SetSample)(unsigned channel, unsigned smpHandle);
    int (CALLING *GetSample)(unsigned channel, unsigned *smpHandle);
    int (CALLING *SetPosition)(unsigned channel, unsigned pos);
    int (CALLING *GetPosition)(unsigned channel, unsigned *pos);
    int (CALLING *GetDirection)(unsigned channel, int *direction);
    int (CALLING *SetPanning)(unsigned channel, int panning);
    int (CALLING *GetPanning)(unsigned channel, int *panning);
    int (CALLING *MuteChannel)(unsigned channel, int mute);
    int (CALLING *AddSample)(sdSample *sample, int copySample,
        unsigned *smpHandle);
    int (CALLING *RemoveSample)(unsigned smpHandle);
    int (CALLING *SetUpdRate)(unsigned updRate);
    int (CALLING *StartPlay)(void);
    int (CALLING *Play)(int *callMP);

#ifdef SUPPORTSTREAMS
    int (CALLING *StartStream)(unsigned channel, uchar *buffer, unsigned
        bufferLength, int sampleType, ulong rate);
    int (CALLING *StopStream)(unsigned channel);
    int (CALLING *SetLoopCallback)(unsigned channel, void (CALLING *callback)
        (unsigned channel));
    int (CALLING *SetStreamWritePosition)(unsigned channel,
        unsigned position);
#endif
} SoundDevice;



#endif


/*
 * $Log: sdevice.h,v $
 * Revision 1.9  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.8  1997/01/16 18:21:23  pekangas
 * Added function SetStreamWritePosition
 *
 * Revision 1.7  1996/10/09 17:59:59  pekangas
 * Fixed to conform to the Sound Device specification
 *
 * Revision 1.6  1996/07/13 18:01:30  pekangas
 * Fixed to compile with Visual C
 *
 * Revision 1.5  1996/05/28 20:28:45  pekangas
 * Fixed a minor but annoying typo
 *
 * Revision 1.4  1996/05/26 20:56:17  pekangas
 * Added rate to SoundDevice.StartStream prototype
 *
 * Revision 1.3  1996/05/24 16:20:36  jpaana
 * Fixed for Linux
 *
 * Revision 1.2  1996/05/23 18:24:02  pekangas
 * Added 8-bit stereo and 16-bit mono & stereo sample types and stream playing functions
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/



//#include "gmplayer.h"
/*      gmplayer.h
 *
 * Generic Module Player
 *
 * $Id: gmplayer.h,v 1.9 1996/10/06 16:48:10 pekangas Exp $
 *
 * Copyright 1996 Petteri Kangaslampi and Jarno Paananen
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#ifndef __GMPLAYER_H
#define __GMPLAYER_H


#define MAXSONGS 16                     /* maximum number of songs played
                                           simultaneously */



/****************************************************************************\
*       enum gmpCommands
*       ----------------
* Description:  Command numbers for module commands
\****************************************************************************/

enum gmpCommands
{
    gmpcNone = 0,                       /* no command */
    gmpcArpeggio = 1,                   /* arpeggio */
    gmpcSlideUp = 2,                    /* period slide up */
    gmpcSlideDown = 3,                  /* period slide down */
    gmpcTonePortamento = 4,             /* tone portamento */
    gmpcVibrato = 5,                    /* vibrato */
    gmpcTPortVSlide = 6,                /* tone portamento + volume slide */
    gmpcVibVSlide = 7,                  /* vibrato + volume slide */
    gmpcTremolo = 8,                    /* tremolo */
    gmpcSetPanning = 9,                 /* set panning (PT cmd 8) */
    gmpcSampleOffset = 10,              /* set sample offset */
    gmpcVolumeSlide = 11,               /* volume slide */
    gmpcPositionJump = 12,              /* position jump */
    gmpcSetVolume = 13,                 /* set volume */
    gmpcPatternBreak = 14,              /* pattern break (to a row) */
    gmpcSetSpeed = 15,                  /* set speed */
    gmpcSetTempo = 16,                  /* set tempo in BPM */
    gmpcFineSlideUp = 17,               /* fine period slide up */
    gmpcFineSlideDown = 18,             /* fine period slide down */
    gmpcPatternLoop = 19,               /* pattern loop set/loop */
    gmpcSetPanning16 = 20,              /* set 16-point panning value */
    gmpcPTRetrig = 21,                  /* Protracker-style retrig note */
    gmpcFineVolSlideUp = 22,            /* fine volume slide up */
    gmpcFineVolSlideDown = 23,          /* fine volume slide down */
    gmpcNoteCut = 24,                   /* note cut */
    gmpcNoteDelay = 25,                 /* note delay */
    gmpcPatternDelay = 26,              /* pattern delay */
    gmpcSetMVolume = 27,                /* set master volume */
    gmpcMVolSlide = 28,                 /* master volume slide */
    gmpcS3MRetrig = 29,                 /* S3M retrig note */
    gmpcMusicSync = 30,                 /* music synchronization command */
    gmpcExtraFineSlideUp = 31,          /* extra fine period slide up */
    gmpcExtraFineSlideDown = 32,        /* extra fine period slide down */
    gmpcPanSlide = 33,                  /* panning slide */
    gmpNumCommands                      /* total number of commands */
/* Hack for Watcom C 10.0: */
#define GMP_NUM_COMMANDS 34
};



/****************************************************************************\
*       enum gmpPlayModes
*       -----------------
* Description:  Generic Module Player playing modes - affects command
*               implementation and other tracker compatibility issues
\****************************************************************************/

enum gmpPlayModes
{
    gmpPT = 1,                          /* Protracker compatibility mode */
    gmpST3,                             /* Scream Tracker 3 mode */
    gmpFT2                              /* Fast Tracker 2 mode */
};




/****************************************************************************\
*       struct gmpSample
*       ----------------
* Description:  Generic Module Player sample in memory
\****************************************************************************/

typedef struct
{
    char        name[32];               /* sample name */
    unsigned    sdHandle;               /* Sound Device sample handle */
    uchar       *sample;                /* sample data pointer, NULL if sample
                                           data only in SD memory */
    unsigned    sampleLength;           /* sample length */
    uchar       sampleType;             /* sample type, see enum
                                           sdSampleType */
    uchar       loopMode;               /* sample looping mode, see enum
                                           sdLoopMode */
    uchar       loop1Type;              /* first loop type, see enum
                                           sdLoopType */
    uchar       loop2Type;              /* second loop type, see enum
                                           sdLoopType */
    unsigned    loop1Start;             /* first loop start */
    unsigned    loop1End;               /* first loop end */
    unsigned    loop2Start;             /* second loop start */
    unsigned    loop2End;               /* second loop end */
    unsigned    volume;                 /* sample volume */
    int         panning;                /* sample panning position */
    long        baseTune;               /* base tune */
    int         finetune;               /* fine tuning value */
    ulong       fileOffset;             /* sample start offset in module
                                           file */
} gmpSample;




/****************************************************************************\
*       struct gmpEnvelope
*       ------------------
* Description:  Envelope information for one instrument envelope (volume or
*               panning)
\****************************************************************************/

typedef struct
{
    uchar       numPoints;              /* number of points in envelope */
    signed char sustain;                /* sustain point number, -1 if no
                                           sustain is used */
    signed char loopStart;              /* envelope loop start point number,
                                           -1 if no loop */
    signed char loopEnd;                /* envelope loop end point number, -1
                                           if no loop */
    signed short points[24];            /* envelope points: a maximum of 12
                                           points, in (X,Y) coordinate
                                           pairs */
} gmpEnvelope;




/****************************************************************************\
*       struct gmpInstrument
*       --------------------
* Description:  Generic Module Player instrument in memory
\****************************************************************************/

typedef struct
{
    char        name[32];               /* instrument name */
    unsigned    numSamples;             /* number of samples */
    uchar       *noteSamples;           /* sample numbers for all keyboard
                                           notes, NULL if sample 0 is used
                                           for all (always 96 bytes) */
    gmpEnvelope *volEnvelope;           /* pointer to volume envelope info or
                                           NULL if volume envelope is not
                                           used */
    gmpEnvelope *panEnvelope;           /* pointer to panning envelope info or
                                           NULL if panning envelope is not
                                           used */
    int         volFadeout;             /* FT2 volume fade-out speed */
    uchar       vibType;                /* FT2 auto-vibrato type */
    uchar       vibSweep;               /* FT2 auto-vibrato sweep value */
    uchar       vibDepth;               /* FT2 auto-vibrato depth */
    uchar       vibRate;                /* FT2 auto-vibrato rate */
    int         used;                   /* 1 if instrument is used, 0 if not.
                                           Unused instruments are not loaded
                                           or added to Sound Device */
    gmpSample   samples[EMPTYARRAY];    /* samples */
} gmpInstrument;




/****************************************************************************\
*       struct gmpPattern
*       -----------------
* Description:  One Generic Module Player pattern in memory
\****************************************************************************/

typedef struct
{
    unsigned    length;                 /* pattern size in bytes, INCLUDING
                                           this header */
    unsigned    rows;                   /* number of rows in pattern */
    uchar       data[EMPTYARRAY];       /* pattern data */

    /* The pattern data is built as follows:
       The first byte is compression information byte. Bits 0-4 form the
       channel number (0-31) and bits 5-7 compression info:
                bit 5   if 1, note and instrument number bytes follow
                bit 6   if 1, volume column byte follows
                bit 7   if 1, command and infobyte bytes follow
       If the compression infobyte is 0, it marks the end of the current
       row. Possible other data follows the compression information byte in
       the following order: note, instrument, volume, command, infobyte.

       In note numbers the upper nybble is the octave number, and lower
       nybble note number in octave. So, for example, D#2 becomes 0x23.
       0xFE as a note number represents a release note, and 0xFF no new
       note as does 0xFF as an instrument number. Valid instrument numbers
       range from 0 to 254 inclusive.

       This isn't quite as efficient as it could be, but is very simple and
       fast to process, and compresses most pattern data very efficiently
       anyway.
    */

} gmpPattern;




/****************************************************************************\
*       struct gmpModule
*       ----------------
* Description:  Generic Module Player module in memory
\****************************************************************************/

typedef struct
{
    uchar       name[32];               /* module name */
    int         playMode;               /* module playing mode - see enum
                                           gmpPlayMode */
    unsigned    songLength;             /* song length */
    unsigned    restart;                /* song restart position */
    unsigned    numInsts;               /* number of instruments */
    unsigned    numPatts;               /* number of patterns */
    uchar       numChannels;            /* number of channels in module */
    uchar       masterVolume;           /* initial master volume */
    uchar       speed;                  /* initial speed */
    uchar       tempo;                  /* initial BPM tempo */
    int         *panning;               /* initial channel panning
                                           positions */
    ushort      *songData;              /* song data - pattern playing
                                           order */
    gmpInstrument **instruments;        /* instrument data */
    gmpPattern  **patterns;             /* pattern data */
    struct  /* Format-specific playing information flags: */
    {
        int     ptLimits : 1;           /* ST3: Protracker limits */
        int     fastVolSlides : 1;      /* ST3: Fast volume slides */
        int     extOctaves : 1;         /* PT: Extended octaves needed */
        int     linFreqTable : 1;       /* FT2: Linear frequency table used */
    } playFlags;
} gmpModule;




/****************************************************************************\
*       struct gmpChannel
*       -----------------
* Description:  Generic Module Player internal channel structure
\****************************************************************************/

typedef struct
{
    unsigned    period;                 /* current playing period */
    int         instrument;             /* current playing instrument, -1 if
                                           no instrument has been set */
    int         newInstrument;          /* new instrument number */
    unsigned    tpDest;                 /* tone portamento destination */
    unsigned    startOffset;            /* next note start offset */

    uchar       sample;                 /* current playing sample, 0xFF if
                                           there is no sample */
    uchar       sdChannel;              /* Sound Device channel number */
    uchar       volColumn;              /* volume column value */
    uchar       note;                   /* current note number, 0xFF if there
                                           is no note */

    int         oldInfo;                /* previous non-zero info byte */

    uchar       oldInfobytes[gmpNumCommands];   /* Old infobytes for all
                                                   commands */
/* Align structure: */
#if (GMP_NUM_COMMANDS % 4)
    uchar       oldInfoFiller[4 - (GMP_NUM_COMMANDS % 4)];
#endif

    uchar       volume;                 /* current volume */
    uchar       command;                /* current command number */
    uchar       infobyte;               /* current command infobyte */
    uchar       tpSpeed;                /* tone portamento speed */

    uchar       realVolume;             /* sound device volume (for envelopes) */
    uchar       volSustained;           /* volume envelope sustained */
    uchar       panSustained;           /* panning envelope sustained */

    uchar       fill;

    int         panning;                /* current panning */

    int         volEnvX;                /* volume envelope X-position */
    int         panEnvX;                /* panning envelope X-position */

    long        fadeOut;                /* fade out value */

    uchar       keyOff;                 /* there has been a key off -note */
    uchar       retrigCount;            /* note retrig counter */
    uchar       trueVolume;             /* ;) */
    uchar       prevNote;               /* previous real note value (the one
                                           that might be playing), 0xFF if
                                           no note has been played */

    uchar       vibSpeed;               /* vibrato speed */
    uchar       vibDepth;               /* vibrato depth */
    uchar       vibPos;                 /* vibrato position */
    uchar       smpOffset;              /* sample offset infobyte */

    int         realPeriod;             /* final period value */
    int         truePeriod;             /* final final period value */

    int         autoVibDepth;           /* auto vibrato depth */
    uchar       autoVibPos;             /* auto vibrato position */

    uchar       loopRow;                /* pattern loop start row (PT mode) */
    uchar       loopCount;              /* pattern loop count (PT mode) */
    uchar       volSlideInfobyte;       /* volume slide old infobyte */

    struct  /* Channel status flag bits: */
    {
        int     newNote : 1;            /* there is a new note number */
        int     newVolume : 1;          /* there is a new volume column
                                           value */
        int     newInst : 1;            /* there is a new instrument number */
        int     command : 1;            /* there is a valid command */
        int     noteDelay : 1;          /* note delay is active */
#ifdef __16__
        int     fillerb : 11;           /* make sure all word fields really */
#else                                   /* do get aligned to word boundaries*/
        int     fillerb : 27;           /* (either 32 or 16 bits) */
#endif
    } status;
} gmpChannel;




/****************************************************************************\
*       struct gmpInformation
*       ---------------------
* Description:  Generic Module Player information structure, used by
*               gmpGetInformation().
\****************************************************************************/

typedef struct
{
    unsigned    position;               /* current playing position */
    unsigned    pattern;                /* current pattern number */
    unsigned    row;                    /* current row in pattern */
    unsigned    tempo;                  /* current tempo in BPM */
    unsigned    speed;                  /* current playing speed */
    int         syncInfo;               /* latest music synchronization
                                           command infobyte or -1 if no
                                           synchronization commands yet */
    gmpChannel  *channels;              /* pointer to channel information
                                           structures. DO NOT MODIFY these! */
} gmpInformation;




/****************************************************************************\
*       struct gmpPlayStatus
*       --------------------
* Description:  Generic Module Player internal playing status structure. Used
*               to enable playing several songs at the same time.
\****************************************************************************/

typedef struct
{
    unsigned    handleNum;              /* playing handle number for this
                                           status struct in GMP internal
                                           table */
    gmpModule   *module;                /* currently playing module */
    uchar       *playPtr;               /* pointer to current pattern data
                                           playing position */
    unsigned    position;               /* current song position */
    unsigned    pattern;                /* current pattern number */
    unsigned    row;                    /* current pattern row number */
    unsigned    songEnd;                /* song end position */
    unsigned    restartPos;             /* song restart position */
    unsigned    tempo;                  /* current tempo in BPM */
    unsigned    speed;                  /* current playing speed */
    unsigned    playCount;              /* playing counter - pattern data is
                                           processed when playCount >= speed
                                         */
    unsigned    pattDelayCount;         /* pattern delay counter */
    unsigned    numChannels;            /* number of channels to be played */
    unsigned    perLimitUp;             /* upper period limit for playing */
    unsigned    perLimitLow;            /* lower period limit for playing */
    unsigned    perMultiplier;          /* multiplier for period values */
    unsigned    volLimit;               /* upper volume limit */

    uchar       loopRow;                /* pattern loop start row */
    uchar       loopCount;              /* pattern loop count */
    uchar       masterVolume;           /* master volume */
    uchar       filler;

    gmpChannel  *channels;              /* GMP channel structures */
    gmpInformation *information;        /* GMP information structure */
    void (CALLING *SyncCallback)(unsigned syncNum, unsigned position,
        unsigned row);                  /* music synchronization callback
                                           function */
    int         syncInfo;               /* latest music synchronization
                                           command infobyte or -1 if no
                                           synchronization commands yet */
} gmpPlayStatus;




/****************************************************************************\
*       typedef gmpPlayHandle
*       ---------------------
* Description:  Generic Module Player playing handle
\****************************************************************************/

typedef gmpPlayStatus* gmpPlayHandle;




#ifdef __cplusplus
extern "C" {
#endif



/****************************************************************************\
*
* Function:     int gmpInit(SoundDevice *SD);
*
* Description:  Initializes Generic Module Player
*
* Input:        SoundDevice *SD         Pointer to the Sound Device that will
*                                       be used for playing the music
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpInit(SoundDevice *SD);




/****************************************************************************\
*
* Function:     int gmpClose(void);
*
* Description:  Uninitializes Generic Module Player
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpClose(void);



/****************************************************************************\
*
* Function:     int gmpSetUpdRateFunct(int (*SetUpdRate)(unsigned updRate));
*
* Description:  Changes the function that will be used to change the playing
*               update rate. Every time song tempo changes, SetUpdRate()
*               will be called with the new player tick rate (in 100*Hz).
*
* Input:        int (*SetUpdRate)(unsigned updRate)   Pointer to updating rate
*                                                     changing function.
*
* Returns:      MIDAS error code
*
* Notes:        SetUpdRate usually points to tmrSetUpdRate(). GMPlayer
*               automatically sets the correct updating rate to the Sound
*               Device used.
*
\****************************************************************************/

int CALLING gmpSetUpdRateFunct(int (CALLING *_SetUpdRate)(unsigned updRate));




/****************************************************************************\
*
* Function:     int gmpPlaySong(gmpModule *module, int startPos, int endPos,
*                   int restartPos, int numChannels, unsigned *sdChannels,
*                   gmpPlayHandle *playHandle)
*
* Description:  Starts playing a song from a module
*
* Input:        gmpModule *module       pointer to module to be played
*               int startPos            song start position
*               int endPos              song end position
*               int restartPos          song restart position
*               int numChannels         number of channels to be played
*               unsigned *sdChannels    pointer to an array of Sound Device
*                                       channel numbers for all channels that
*                                       are required for playing
*               gmpPlayHandle *playHandle   pointer to GMP playing handle
*
* Returns:      MIDAS error code. Playing handle for this song is written to
*               *playHandle.
*
* Notes:        To play the whole module, set startPos, endPos, restartPos and
*               numChannels to -1.
*
\****************************************************************************/

int CALLING gmpPlaySong(gmpModule *module, int startPos, int endPos,
    int restartPos, int numChannels, unsigned *sdChannels,
    gmpPlayHandle *playHandle);




/****************************************************************************\
*
* Function:     int gmpStopSong(gmpPlayHandle playHandle)
*
* Description:  Stops playing a song
*
* Input:        gmpPlayHandle playHandle   playing handle returned by
*                                          gmpPlaySong().
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpStopSong(gmpPlayHandle playHandle);




/****************************************************************************\
*
* Function:     int gmpPlay(void)
*
* Description:  Plays music for one song tick.
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int CALLING gmpPlay(void);




/****************************************************************************\
*
* Function:     int gmpFreeModule(gmpModule *module)
*
* Description:  Deallocates a module structure allocated by a module loader
*
* Input:        gmpModule *module       module to be deallocated
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpFreeModule(gmpModule *module);




/****************************************************************************\
*
* Function:     int gmpLoadMOD(char *fileName, int addSD, mpModule **module)
*
* Description:  Loads a Protracker module to memory in Generic Module Player
*               module format
*
* Input:        char *fileName          module file name
*               int addSD               1 if module samples should be added to
*                                       the current Sound Device, 0 if not
*               int (*SampleCallback)(...)  pointer to callback function that
*                                       will be called after sample has been
*                                       added to Sound Device, but before
*                                       sample data is deallocated
*               mpModule **module       pointer to GMP module pointer
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpLoadMOD(char *fileName, int addSD, int
    (CALLING *SampleCallback)(sdSample *sdsmp, gmpSample *gmpsmp),
    gmpModule **_module);




/****************************************************************************\
*
* Function:     int gmpGetInformation(gmpPlayHandle playHandle,
*                   gmpInformation **information);
*
* Description:  Reads current playing status on a playing handle
*
* Input:        gmpPlayHandle playHandle        playing handle
*               gmpInformation **information    pointer to pointer to
*                                               information structure
*
* Returns:      MIDAS error code. Pointer to GMP information structure filled
*               is written to *information.
*
\****************************************************************************/

int CALLING gmpGetInformation(gmpPlayHandle playHandle,
    gmpInformation **information);




/****************************************************************************\
*
* Function:     int gmpSetTempo(unsigned tempo)
*
* Description:  Changes current playing tempo. GMP internal use only. Updates
*               Sound Device and UpdRateFunct() update rates as necessary.
*
* Input:        unsigned tempo          new tempo in beats per minute
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int gmpSetTempo(unsigned tempo);




/****************************************************************************\
*
* Function:     int gmpNotePeriod(unsigned note, unsigned *period)
*
* Description:  Converts a note number to a period value (both depend on
*               current playing mode). Uses *gmpChan and *gmpPlayModule to
*               get sample tuning values. GMP internal.
*
* Input:        unsigned note           note number
*               unsigned *period        pointer to period value
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpNotePeriod(unsigned note, unsigned *period);




/****************************************************************************\
*
* Function:     int gmpPlayNote(unsigned period)
*
* Description:  Starts playing a new note on channel *gmpChan with period
*               value period. GMP internal.
*
* Input:        unsigned period         period value for new note
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpPlayNote(unsigned period);




/****************************************************************************\
*
* Function:     int gmpPeriodRate(unsigned period, ulong *rate)
*
* Description:  Converts a period value to sampling rate, depending on current
*               playing mode. GMP internal.
*
* Input:        unsigned period         period number
*               ulong *rate             pointer to sampling rate
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int gmpPeriodRate(unsigned period, ulong *rate);




/****************************************************************************\
*
* Function:     int gmpSetPeriod(unsigned period)
*
* Description:  Sets the playing period on current channel. Updates the
*               value in *gmpChan and sets it to Sound Device, taking
*               current period limits into account.
*
* Input:        unsigned period         new period value
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpSetPeriod(unsigned period);




/****************************************************************************\
*
* Function:     int gmpChangePeriod(unsigned period)
*
* Description:  Changes the playing period on current channel. Sets the new
*               value to Sound Device, but does NOT update channel structure.
*
* Input:        unsigned period         new period value
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpChangePeriod(unsigned period);




/****************************************************************************\
*
* Function:     int gmpSetVolume(int volume)
*
* Description:  Sets the playing volume on current channel. Sets the volume
*               to Sound Device and updates the channel structure, taking
*               current volume limits into account
*
* Input:        int volume              new playing volume (signed to allow
*                                       easier limit checking)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpSetVolume(int volume);




/****************************************************************************\
*
* Function:     int gmpChangeVolume(int volume)
*
* Description:  Changes the playing volume on current channel. Sets the
*               volume to taking current volume limits into account, but
*               does NOT update channel structures.
*
* Input:        int volume              new playing volume (signed to allow
*                                       easier limit checking)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpChangeVolume(int volume);


/****************************************************************************\
*
* Function:     int gmpSetPanning(int panning)
*
* Description:  Sets the panning of current channel. Sets the panning
*               to Sound Device and updates the channel structure, taking
*               current panning limits into account
*
* Input:        int panning             new panning
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpSetPanning(int panning);


/****************************************************************************\
*
* Function:     int gmpNewNote(void)
*
* Description:  Plays the new note on the current channel
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpNewNote(void);




/****************************************************************************\
*
* Function:     int gmpLoadXM(char *fileName, int addSD, mpModule **module)
*
* Description:  Loads a Fasttracker 2 module to memory in Generic Module
*               Player module format
*
* Input:        char *fileName          module file name
*               int addSD               1 if module samples should be added to
*                                       the current Sound Device, 0 if not
*               int (*SampleCallback)(...)  pointer to callback function that
*                                       will be called after sample has been
*                                       added to Sound Device, but before
*                                       sample data is deallocated
*               mpModule **module       pointer to GMP module pointer
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpLoadXM(char *fileName, int addSD, int
    (CALLING *SampleCallback)(sdSample *sdsmp, gmpSample *gmpsmp),
    gmpModule **_module);




/****************************************************************************\
*
* Function:     int gmpLoadS3M(char *fileName, int addSD, mpModule **module)
*
* Description:  Loads a Scream Tracker 3 module to memory in Generic Module
*               Player module format
*
* Input:        char *fileName          module file name
*               int addSD               1 if module samples should be added to
*                                       the current Sound Device, 0 if not
*               int (*SampleCallback)(...)  pointer to callback function that
*                                       will be called after sample has been
*                                       added to Sound Device, but before
*                                       sample data is deallocated
*               mpModule **module       pointer to GMP module pointer
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpLoadS3M(char *fileName, int addSD, int
    (CALLING *SampleCallback)(sdSample *sdsmp, gmpSample *gmpsmp),
    gmpModule **_module);




/****************************************************************************\
*
* Function:     int gmpSetVolCommand(void)
*
* Description:  Runs tick-0 volume column command for current channel (FT2)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpSetVolCommand(void);




/****************************************************************************\
*
* Function:     int gmpRunVolCommand(void)
*
* Description:  Runs continuous volume column command for current channel
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int gmpRunVolCommand(void);




/****************************************************************************\
*
* Function:     int gmpSetSyncCallback(void (CALLING *musicSync)(unsigned syncNum, unsigned position, unsigned row))
*
* Description:  Sets music synchronization callback function. This function
*               will be called when the music synchronization command (command
*               W in FT2 and ST3) is encountered in the music. The function
*               receives as arguments the synchronization command infobyte,
*               current playing position position and current row number.
*               Note that the function will be called INSIDE THE PLAYER
*               TIMER INTERRUPT and thus SS != DS !!!
*
* Input:        gmpPlayHandle playHandle    playing handle
*               void (CALLING *SyncCallback)()  Pointer to music
*                                       synchronization callback function.
*                                       Set to NULL to disable callback
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpSetSyncCallback(gmpPlayHandle playHandle,
    void (CALLING *SyncCallback)(unsigned syncNum, unsigned position,
    unsigned row));




/****************************************************************************\
*
* Function:     int gmpSetPosition(gmpPlayHandle playHandle, unsigned
*                   newPosition);
*
* Description:  Changes song playing position.
*
* Input:        gmpPlayHandle playHandle    playing handle
*               unsigned newPosition    new playing position
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING gmpSetPosition(gmpPlayHandle playHandle, int newPosition);




/****************************************************************************\
*      Global variables used by all GMP modules:
\****************************************************************************/


extern SoundDevice * GLOBALVAR gmpSD;   /* Sound Device used by GMP */
extern gmpChannel * GLOBALVAR gmpChan;  /* current GMP channel */
extern unsigned GLOBALVAR gmpTempo;     /* GMP playing tempo (global for
                                           all songs) */
extern unsigned GLOBALVAR gmpPlayMode;  /* current playing mode */
extern gmpPlayHandle GLOBALVAR gmpHandle;   /* current playing handle */
extern gmpModule * GLOBALVAR gmpCurModule;  /* current playing module */




/****************************************************************************\
*      Command pointer tables:
\****************************************************************************/

    /* Protracker playing mode tick-0 commands: */
extern int GLOBALVAR (*gmpTick0CommandsPT[])(unsigned infobyte);

    /* Protracker playing mode continuous commands: */
extern int GLOBALVAR (*gmpContCommandsPT[gmpNumCommands])(unsigned infobyte);

    /* Fasttracker 2 playing mode tick-0 commands: */
extern int GLOBALVAR (*gmpTick0CommandsFT2[])(unsigned infobyte);

    /* Fasttracker 2 playing mode continuous commands: */
extern int GLOBALVAR (*gmpContCommandsFT2[gmpNumCommands])(unsigned infobyte);

    /* Screamtracker 3 playing mode tick-0 commands: */
extern int GLOBALVAR (*gmpTick0CommandsST3[])(unsigned infobyte);

    /* Screamtracker 3 playing mode continuous commands: */
extern int GLOBALVAR (*gmpContCommandsST3[gmpNumCommands])(unsigned infobyte);



#ifdef __cplusplus
}
#endif







/****************************************************************************\
*       enum gmpFunctIDs
*       ----------------
* Description:  ID numbers for Generic Module Player
\****************************************************************************/

enum gmpFunctIDs
{
    ID_gmpInit = ID_gmp,
    ID_gmpClose,
    ID_gmpSetUpdRateFunct,
    ID_gmpPlaySong,
    ID_gmpStopSong,
    ID_gmpPlay,
    ID_gmpSetTempo,
    ID_gmpPlayPattern,
    ID_gmpHandleCommands,
    ID_gmpNotePeriod,
    ID_gmpPlayNote,
    ID_gmpPeriodRate,
    ID_gmpFreeModule,
    ID_gmpLoadMOD,
    ID_gmpSetPeriod,
    ID_gmpChangePeriod,
    ID_gmpSetVolume,
    ID_gmpChangeVolume,
    ID_gmpNewNote,
    ID_gmpLoadXM,
    ID_gmpRunEnvelopes,
    ID_gmpLoadS3M,
    ID_gmpSetPanning,
    ID_gmpSetSyncCallback
};



#endif


/*
 * $Log: gmplayer.h,v $
 * Revision 1.9  1996/10/06 16:48:10  pekangas
 * Fixed a overflow problem in panning
 *
 * Revision 1.8  1996/09/01 18:33:34  pekangas
 * Added GMP_NUM_COMMANDS #define, hack for Watcom C 10.0
 *
 * Revision 1.7  1996/09/01 15:41:43  pekangas
 * Changed command handling for FT2 and splitted some slides in two commands
 *
 * Revision 1.6  1996/07/13 20:01:05  pekangas
 * Changed gmpInstrument.numsamples to unsigned
 *
 * Revision 1.5  1996/07/13 19:41:13  pekangas
 * Eliminated Visual C warnings
 *
 * Revision 1.4  1996/07/13 18:04:50  pekangas
 * Fixed to compile with Visual C
 *
 * Revision 1.3  1996/05/24 16:59:11  pekangas
 * Fixed to work with Watcom C again - using EMPTYARRAY
 *
 * Revision 1.2  1996/05/24 16:20:36  jpaana
 * Fixed to work with gcc
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/



//#include "timer.h"
/*      TIMER.H
 *
 * TempoTimer
 *
 * $Id: timer.h,v 1.3 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __TIMER_H
#define __TIMER_H

#ifdef __cplusplus
extern "C" {
#endif


/****************************************************************************\
*
* Function:     int tmrGetScrSync(unsigned *scrSync);
*
* Description:  Calculates the screen synchronization value for timer
*
* Input:        unsigned *scrSync       pointer to screen synchronization
*                                       value
*
* Returns:      MIDAS error code.
*               Screen syncronization value used with tmrSyncScr() is stored
*               in *scrSync.
*
\****************************************************************************/

int CALLING tmrGetScrSync(unsigned *scrSync);




/****************************************************************************\
*
* Function:     int tmrInit(void);
*
* Description:  Initializes TempoTimer.
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING tmrInit(void);




/****************************************************************************\
*
* Function:     int tmrClose(void);
*
* Description:  Uninitializes TempoTimer. MUST be called if and ONLY if
*               tmrInit() has been called.
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING tmrClose(void);




/****************************************************************************\
*
* Function:     int tmrPlaySD(SoundDevice *SD);
*
* Description:  Starts playing sound with a Sound Device ie. calling its
*               Play() function in the update rate, which is set to
*               50Hz.
*
* Input:        SoundDevice *SD         Sound Device that will be used
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int CALLING tmrPlaySD(SoundDevice *SD);




/****************************************************************************\
*
* Function:     int tmrStopSD(void);
*
* Description:  Stops playing sound with the Sound Device.
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int CALLING tmrStopSD(void);




/****************************************************************************\
*
* Function:     int tmrPlayMusic(void *play, int *playerNum);
*
* Description:  Starts playing music with the timer.
*
* Input:        void *play              Pointer to music playing function,
*                                       must return MIDAS error codes
*               int *playerNum          Pointer to player number, used
*                                       for stopping music
*
* Returns:      MIDAS error code. Player number is written to *playerNum.
*
* Notes:        There can be a maximum of 16 music players active at the
*               same time.
*
\****************************************************************************/

int CALLING tmrPlayMusic(int (CALLING *play)(), int *playerNum);




/****************************************************************************\
*
* Function:     int tmrStopMusic(int playerNum);
*
* Description:  Stops playing music with the timer.
*
* Input:        int playerNum           Number of player to be stopped.
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING tmrStopMusic(int playerNum);




/****************************************************************************\
*
* Function:     int tmrSyncScr(unsigned sync, void (*preVR)(),
*                   void (*immVR)(), void (*inVR)());
*
* Description:  Synchronizes the timer to screen refresh.
*
* Input:        unsigned sync           Screen synchronization value returned
*                                       by tmrGetScrSync().
*               void (*preVR)()         Pointer to the routine that will be
*                                       called BEFORE Vertical Retrace
*               void (*immVR)()         Pointer to the routine that will be
*                                       called immediately after Vertical
*                                       Retrace starts
*               void (*inVR)()          Pointer to the routine that will be
*                                       called some time during Vertical
*                                       Retrace
*
* Returns:      MIDAS error code
*
* Notes:        preVR() and immVR() functions must be as short as possible
*               and do nothing else than update counters or set some VGA
*               registers to avoid timer synchronization problems. inVR()
*               can take a longer time and can be used for, for example,
*               setting the palette.
*
*               Remember to use the correct calling convention for the xxVR()
*               routines! (pascal for Pascal programs, cdecl otherwise).
*
\****************************************************************************/

int CALLING tmrSyncScr(unsigned sync, void (CALLING *preVR)(),
    void (CALLING *immVR)(), void (CALLING *inVR)());




/****************************************************************************\
*
* Function:     int tmrStopScrSync(void);
*
* Description:  Stops synchronizing the timer to the screen.
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING tmrStopScrSync(void);




/****************************************************************************\
*
* Function:     int tmrSetUpdRate(unsigned updRate);
*
* Description:  Sets the timer update rate, ie. the rate at which the music
*               playing routines are called
*
* Input:        unsigned updRate        updating rate, in 100*Hz (5000=50Hz)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING tmrSetUpdRate(unsigned updRate);


#ifdef __cplusplus
}
#endif


/****************************************************************************\
*       enum tmrFunctIDs
*       ----------------
* Description:  ID numbers for TempoTimer functions
\****************************************************************************/

enum tmrFunctIDs
{
    ID_tmrGetScrSync = ID_tmr,
    ID_tmrInit,
    ID_tmrClose,
    ID_tmrPlaySD,
    ID_tmrStopSD,
    ID_tmrPlayMusic,
    ID_tmrStopMusic,
    ID_tmrSyncScr,
    ID_tmrStopScrSync,
    ID_tmrSetUpdRate
};



#endif


/*
 * $Log: timer.h,v $
 * Revision 1.3  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.2  1996/07/13 18:09:09  pekangas
 * Fixed to compile with Visual C
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/



//#include "dma.h"
/*      DMA.H
 *
 * DMA handling routines
 *
 * $Id: dma.h,v 1.2 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __DMA_H
#define __DMA_H



/****************************************************************************\
*       struct dmaBuffer
*       ----------------
* Description:  DMA playing buffer
\****************************************************************************/

typedef struct
{
#ifdef __16__
    unsigned    bufferSeg;              /* DMA buffer segment (offset is
                                           zero) */
#endif
#ifdef __PROTMODE__
    unsigned    dosSeg;                 /* DMA buffer DOS segment */
    unsigned    dpmiSel;                /* DMA buffer DPMI selector */
#endif
    ulong       startAddr;              /* buffer physical start address */
    unsigned    bufferLen;              /* DMA buffer length in bytes */
    void        *memBlk;                /* internal, used for unallocating */
    int         channel;                /* channel on which the buffer is
                                           being played or -1 */
#if defined(__PROTMODE__) && !defined(__FLATMODE__)
    void far    *dataPtr;               /* pointer to DMA buffer data */
#else
    void        *dataPtr;               /* pointer to DMA buffer data *//*!!*/
#endif
} dmaBuffer;




/****************************************************************************\
*       struct dmaChannel
*       -----------------
* Description:  DMA channel data used by DMA functions.
\****************************************************************************/

typedef struct
{
    unsigned    number;                 /* DMA channel number */
    unsigned    bit;                    /* DMA channel bit for registers */
    unsigned    baseAddr;               /* base address port */
    unsigned    wordCount;              /* word count port */
    unsigned    request;                /* request port */
    unsigned    singleMask;             /* mask port */
    unsigned    mode;                   /* mode port */
    unsigned    clearFF;                /* clear flip-flop port */
    unsigned    page;                   /* page port */
} dmaChannel;



#ifdef __cplusplus
extern "C" {
#endif



/****************************************************************************\
*
* Function:     int dmaAllocBuffer(unsigned size, dmaBuffer *buf);
*
* Description:  Allocates a DMA buffer (totally inside a 64K physical page)
*
* Input:        unsigned size           size of buffer in bytes
*               dmaBuffer *buf          pointer to DMA buffer information
*
* Returns:      MIDAS error code. DMA buffer information is written to *buf.
*
\****************************************************************************/

int CALLING dmaAllocBuffer(unsigned size, dmaBuffer *buf);




/****************************************************************************\
*
* Function:     int dmaFreeBuffer(dmaBuffer *buf);
*
* Description:  Deallocates an allocated DMA buffer
*
* Input:        dmaBuffer *buf          pointer to DMA buffer information
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dmaFreeBuffer(dmaBuffer *buf);




/****************************************************************************\
*
* Function:     int dmaPlayBuffer(dmaBuffer *buf, unsigned channel,
*                   unsigned autoInit);
*
* Description:  Plays a DMA buffer
*
* Input:        dmaBuffer *buf          pointer to DMA buffer information
*               unsigned channel        DMA channel number
*               unsigned autoInit       1 if autoinitializing DMA is used, 0
*                                       if not
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dmaPlayBuffer(dmaBuffer *buf, unsigned channel,
    unsigned autoInit);




/****************************************************************************\
*
* Function:     int dmaStop(unsigned channel);
*
* Description:  Stops DMA playing
*
* Input:        unsigned channel        DMA channel number
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dmaStop(unsigned channel);




/****************************************************************************\
*
* Function:     int dmaGetPos(dmaBuffer *buf, unsigned *pos);
*
* Description:  Reads the DMA playing position
*
* Input:        dmaBuffer *buf          pointer to DMA buffer information
*               unsigned *pos           pointer to playing position
*
* Returns:      MIDAS error code. DMA playing position from the beginning
*               of the buffer, in bytes, is written to *pos.
*
\****************************************************************************/

int CALLING dmaGetPos(dmaBuffer *buf, unsigned *pos);




#ifdef __cplusplus
}
#endif




/****************************************************************************\
*       enum dmaFunctIDs
*       ----------------
* Description:  ID numbers for DMA handling functions
\****************************************************************************/

enum dmaFunctIDs
{
    ID_dmaAllocBuffer = ID_dma,
    ID_dmaFreeBuffer,
    ID_dmaPlayBuffer,
    ID_dmaStop,
    ID_dmaGetPos
};


#endif


/*
 * $Log: dma.h,v $
 * Revision 1.2  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/



//#include "dsm.h"
/*      DSM.H
 *
 * Digital Sound Mixer
 *
 * $Id: dsm.h,v 1.7 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#ifndef __DSM_H
#define __DSM_H


#define VOLLEVELS 33
#define VOLSHIFT 1

/* Magic sample handle for streams: */
#define DSM_SMP_STREAM MAXSAMPLES


//#include "sdevice.h"


/****************************************************************************\
*       enum dsmMixMode
*       ---------------
* Description:  Available DSM mixing modes
\****************************************************************************/

enum dsmMixMode
{
    dsmMixMono = 1,                     /* mono mixing */
    dsmMixStereo = 2                    /* stereo mixing */
};




/****************************************************************************\
*       enum dsmChanStatus
*       ------------------
* Description:  DSM channel sound playing status
\****************************************************************************/

enum dsmChanStatus
{
    dsmChanStopped = 0,                 /* playing is stopped */
    dsmChanEnd,                         /* playing has ended (not forced
                                           stopped */
    dsmChanPlaying,                     /* playing, not released */
    dsmChanReleased                     /* playing, note has been released */
};




/****************************************************************************\
*       enum dsmPlayDir
*       ---------------
* Description:  Playing direction in bidirectional loops
\****************************************************************************/

enum dsmPlayDir
{
    dsmPlayBackwards = -1,              /* playing backwards */
    dsmPlayForward = 1                  /* playing forward */
};




/****************************************************************************\
*       struct dsmChannel
*       -----------------
* Description:  DSM channel data
\****************************************************************************/

typedef struct
{
    uchar       *sample;                /* sample data pointer */
    int         sampleType;             /* sample type, see enum
                                           sdSampleType */
    int         samplePos;              /* sample position in memory */
    unsigned    sampleLength;           /* sample length */
    int         loopMode;               /* sample looping mode, see enum
                                           sdLoopMode */
    unsigned    loop1Start;             /* first loop start */
    unsigned    loop1End;               /* first loop end */
    int         loop1Type;              /* first loop type, see enum
                                           sdLoopType */
    unsigned    loop2Start;             /* second loop start */
    unsigned    loop2End;               /* second loop end */
    int         loop2Type;              /* second loop type, see enum
                                           sdLoopType */

    unsigned    playPos;                /* playing position whole part */
    unsigned    playPosLow;             /* playing position fractional part
                                           (only lower 16 bits used) */
    unsigned    streamWritePos;         /* stream write position */
    ulong       rate;                   /* playing rate in Hz */
    int         direction;              /* playing direction in bidirectional
                                           loops - 1 is forward, -1 back */
    unsigned    sampleHandle;           /* sample handle */
    unsigned    sampleChanged;          /* 1 if sample has been changed
                                           but values not yet set in
                                           channel struct */
    int         panning;                /* panning information */
    unsigned    volume;                 /* playing volume (0-64) */
    int         muted;                  /* 1 if channel muted, 0 if not */
    int         status;                 /* channel status, see enum dsm
                                           dsmChanStatus */
    int         loopNum;                /* currently played sample loop */
    void (CALLING *LoopCallback)(unsigned channel);     /* sample loop
                                                           callback */
} dsmChannel;




/****************************************************************************\
*       struct dsmSample
*       ----------------
* Description:  DSM internal sample structure
\****************************************************************************/

typedef struct
{
    uchar       *sample;                /* sample data pointer */
    int         sampleType;             /* sample type, see enum
                                           sdSampleType */
    int         samplePos;              /* sample position in memory */
    unsigned    sampleLength;           /* sample length */
    int         loopMode;               /* sample looping mode, see enum
                                           sdLoopMode */
    unsigned    loop1Start;             /* first loop start */
    unsigned    loop1End;               /* first loop end */
    int         loop1Type;              /* first loop type, see enum
                                           sdLoopType */
    unsigned    loop2Start;             /* second loop start */
    unsigned    loop2End;               /* second loop end */
    int         loop2Type;              /* second loop type, see enum
                                           sdLoopType */
    int         inUse;                  /* 1 if sample is in use, 0 if not
                                           (removed using dsmRemoveSample) */
    int         copied;                 /* 1 if a copied sample (should be
                                           deallocated), 0 if not */
} dsmSample;



#ifdef __cplusplus
extern "C" {
#endif




extern unsigned * GLOBALVAR dsmMixBuffer;  /* DSM mixing buffer. dsmPlay()
                                           writes the mixed data here. Post-
                                           processing is usually necessary. */
extern unsigned GLOBALVAR dsmMixBufferSize;      /* DSM mixing buffer size */

/* The following global variables are used internally by different DSM
   functions and should not be accessed by other modules: */

extern unsigned GLOBALVAR dsmMixRate;   /* mixing rate in Hz */
extern unsigned GLOBALVAR dsmMode;      /* output mode (see enum
                                           dsmMixMode) */
#ifdef __16__
extern unsigned GLOBALVAR dsmVolTableSeg;   /* volume table segment */
#endif
extern unsigned GLOBALVAR *dsmVolumeTable;  /* pointer to volume table */

extern dsmChannel * GLOBALVAR dsmChannels;  /* pointer to channel datas */
extern dsmSample * GLOBALVAR dsmSamples;    /* sample structures */
extern unsigned GLOBALVAR dsmOutputBits;    /* output bit width */




/****************************************************************************\
*
* Function:     int dsmInit(unsigned mixRate, unsigned mode,
*                   unsigned outputBits);
*
* Description:  Initializes Digital Sound Mixer
*
* Input:        unsigned mixRate        mixing rate in Hz
*               unsigned mode           mixing mode (see enum dsmMixMode)
*               unsigned outputBits     output bit width (if less than
*                                       16, output values are divided
*                                       accordingly - mixing buffer is
*                                       always a sequence of unsigned ints)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmInit(unsigned mixRate, unsigned mode, unsigned amplitude);




/****************************************************************************\
*
* Function:     int dsmClose(void)
*
* Description:  Uninitializes Digital Sound Mixer
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmClose(void);




/****************************************************************************\
*
* Function:     int dsmGetMixRate(unsigned *mixRate)
*
* Description:  Reads the actual mixing rate
*
* Input:        unsigned *mixRate       pointer to mixing rate variable
*
* Returns:      MIDAS error code.
*               Mixing rate, in Hz, is stored in *mixRate
*
\****************************************************************************/

int CALLING dsmGetMixRate(unsigned *mixRate);




/****************************************************************************\
*
* Function:     int dsmOpenChannels(unsigned channels);
*
* Description:  Opens channels for output
*
* Input:        unsigned channels       number of channels to open
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmOpenChannels(unsigned channels);




/****************************************************************************\
*
* Function:     int dsmCalcVolTable(unsigned amplification)
*
* Description:  Calculates a new volume table
*
* Input:        unsigned amplification  Amplification level. 64 - normal
*                                       (100%), 32 = 50%, 128 = 200% etc.
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmCalcVolTable(unsigned amplification);




/****************************************************************************\
*
* Function:     int dsmCloseChannels(void)
*
* Description:  Closes open output channels
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmCloseChannels(void);




/****************************************************************************\
*
* Function:     int dsmClearChannels(void)
*
* Description:  Clears open channels (removes all sounds)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmClearChannels(void);




/****************************************************************************\
*
* Function:     int dsmMute(int mute)
*
* Description:  Mutes all channels
*
* Input:        int mute                1 = mute, 0 = un-mute
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmMute(int mute);




/****************************************************************************\
*
* Function:     int dsmPause(int pause)
*
* Description:  Pauses or resumes playing
*
* Input:        int pause               1 = pause, 0 = resume
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmPause(int pause);




/****************************************************************************\
*
* Function:     int dsmSetMasterVolume(unsigned masterVolume)
*
* Description:  Sets the master volume
*
* Input:        unsigned masterVolume   master volume (0 - 64)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetMasterVolume(unsigned masterVolume);




/****************************************************************************\
*
* Function:     int dsmGetMasterVolume(unsigned *masterVolume)
*
* Description:  Reads the master volume
*
* Input:        unsigned *masterVolume  pointer to master volume
*
* Returns:      MIDAS error code. Master volume is written to *masterVolume.
*
\****************************************************************************/

int CALLING dsmGetMasterVolume(unsigned *masterVolume);




/****************************************************************************\
*
* Function:     int dsmSetAmplification(unsigned amplification)
*
* Description:  Sets amplification level and calculates new volume table.
*
* Input:        unsigned amplification  amplification level, 64 = normal
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetAmplification(unsigned amplification);




/****************************************************************************\
*
* Function:     int dsmGetAmplification(unsigned *amplification)
*
* Description:  Reads the amplification level
*
* Input:        unsigned *amplification   pointer to amplification level
*
* Returns:      MIDAS error code. Amplification level is written to
*               *amplification.
*
\****************************************************************************/

int CALLING dsmGetAmplification(unsigned *amplification);




/****************************************************************************\
*
* Function:     int dsmPlaySound(unsigned channel, ulong rate)
*
* Description:  Starts playing a sound
*
* Input:        unsigned channel        channel number
*               ulong rate              playing rate in Hz
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmPlaySound(unsigned channel, ulong rate);




/****************************************************************************\
*
* Function:     int dsmReleaseSound(unsigned channel)
*
* Description:  Releases the current sound from the channel. If sdLoop1Rel or
*               sdLoop2 looping modes are used, playing will be continued from
*               the release part of the current sample (data after the end
*               of the first loop) after the end of the first loop is reached
*               next time, otherwise the sound will be stopped.
*
* Input:        unsigned channel        channel number
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmReleaseSound(unsigned channel);




/****************************************************************************\
*
* Function:     int dsmStopSound(unsigned channel)
*
* Description:  Stops playing a sound
*
* Input:        unsigned channel        channel number
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmStopSound(unsigned channel);




/****************************************************************************\
*
* Function:     int dsmSetRate(unsigned channel, ulong rate)
*
* Description:  Sets the playing rate
*
* Input:        unsigned channel        channel number
*               ulong rate              playing rate in Hz
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetRate(unsigned channel, ulong rate);




/****************************************************************************\
*
* Function:     int dsmGetRate(unsigned channel, ulong *rate)
*
* Description:  Reads the playing rate on a channel
*
* Input:        unsigned channel        channel number
*               ulong *rate             pointer to playing rate
*
* Returns:      MIDAS error code. Playing rate is written to *rate, 0 if
*               no sound is being played.
*
\****************************************************************************/

int CALLING dsmGetRate(unsigned channel, ulong *rate);




/****************************************************************************\
*
* Function:     int dsmSetVolume(unsigned channel, unsigned volume)
*
* Description:  Sets the playing volume
*
* Input:        unsigned channel        channel number
*               unsigned volume         playing volume (0-64)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetVolume(unsigned channel, unsigned volume);




/****************************************************************************\
*
* Function:     int dsmGetVolume(unsigned channel, unsigned *volume)
*
* Description:  Reads the playing volume
*
* Input:        unsigned channel        channel number
*               unsigned *volume        pointer to volume
*
* Returns:      MIDAS error code. Playing volume is written to *volume.
*
\****************************************************************************/

int CALLING dsmGetVolume(unsigned channe, unsigned *volume);




/****************************************************************************\
*
* Function:     int dsmSetSample(unsigned channel, unsigned smpHandle)
*
* Description:  Sets the sample number on a channel
*
* Input:        unsigned channel        channel number
*               unsigned smpHandle      sample handle returned by
*                                       dsmAddSample()
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetSample(unsigned channel, unsigned smpHandle);




/****************************************************************************\
*
* Function:     int dsmGetSample(unsigned channel, unsigned *smpHandle)
*
* Description:  Reads current sample handle
*
* Input:        unsigned channel        channel number
*               unsigned *smpHandle     pointer to sample handle
*
* Returns:      MIDAS error code. Sample handle is written to *smpHandle;
*
\****************************************************************************/

int CALLING dsmGetSample(unsigned channel, unsigned *smpHandle);




/****************************************************************************\
*
* Function:     int dsmChangeSample(unsigned channel)
*
* Description:  Changes the sample used in a channel to the one specified
*               by the channel's sample handle. Used only internally by
*               other DSM functions, does no error checking.
*
* Input:        unsigned channel        channel number
*
* Returns:      MIDAS error code (does not fail)
*
\****************************************************************************/

int CALLING dsmChangeSample(unsigned channel);




/****************************************************************************\
*
* Function:     int dsmSetPosition(unsigned channel, unsigned position)
*
* Description:  Sets the playing position from the beginning of the sample
*
* Input:        unsigned channel        channel number
*               unsigned position       new playing position
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetPosition(unsigned channel, unsigned position);




/****************************************************************************\
*
* Function:     int dsmGetPosition(unsigned channel, unsigned *position)
*
* Description:  Reads the current playing position
*
* Input:        unsigned channel        channel number
*               unsigned *position      pointer to playing position
*
* Returns:      MIDAS error code. Playing position is written to *position.
*
\****************************************************************************/

int CALLING dsmGetPosition(unsigned channel, unsigned *position);




/****************************************************************************\
*
* Function:     int dsmGetDirection(unsigned channel, int *direction)
*
* Description:  Reads current playing direction
*
* Input:        unsigned channel        channel number
*               int *direction          pointer to playing direction. 1 is
*                                       forward, -1 backwards
*
* Returns:      MIDAS error code. Playing direction is written to *direction.
*
\****************************************************************************/

int CALLING dsmGetDirection(unsigned channel, int *direction);




/****************************************************************************\
*
* Function:     int dsmSetPanning(unsigned channel, int panning)
*
* Description:  Sets the panning position of a channel
*
* Input:        unsigned channel        channel number
*               int panning             panning position (see enum sdPanning)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetPanning(unsigned channel, int panning);




/****************************************************************************\
*
* Function:     int dsmGetPanning(unsigned channel, int *panning)
*
* Description:  Reads the panning position of a channel
*
* Input:        unsigned channel        channel number
*               int *panning            pointer to panning position
*
* Returns:      MIDAS error code. Panning position is written to *panning.
*
\****************************************************************************/

int CALLING dsmGetPanning(unsigned channel, int *panning);




/****************************************************************************\
*
* Function:     int dsmMuteChannel(unsigned channel, int mute)
*
* Description:  Mutes/un-mutes a channel
*
* Input:        unsigned channel        channel number
*               int mute                muting status - 1 = mute, 0 = un-mute
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmMuteChannel(unsigned channel, int mute);




/****************************************************************************\
*
* Function:     int dsmAddSample(sdSample *sample, int copySample,
*                   unsigned *smpHandle);
*
* Description:  Adds a new sample to the DSM sample list and prepares it for
*               DSM use
*
* Input:        sdSample *sample        pointer to sample information
*                                           structure
*               int copySample          copy sample data to a new place in
*                                       memory? 1 = yes, 0 = no
*               unsigned *smpHandle     pointer to sample handle
*
* Returns:      MIDAS error code. Sample handle for the new sample is written
*               to *smpHandle
*
* Notes:        If copySample = 1, sample data must not be in EMS memory
*
\****************************************************************************/

int CALLING dsmAddSample(sdSample *sample, int copySample, unsigned *smpHandle);




/****************************************************************************\
*
* Function:     int dsmRemoveSample(unsigned smpHandle)
*
* Description:  Removes a sample from the sample list and deallocates it if
*               necessary.
*
* Input:        unsigned smpHandle      sample handle returned by
*                                       dsmAddSample()
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmRemoveSample(unsigned smpHandle);




/****************************************************************************\
*
* Function:     int dsmMixData(unsigned numElems)
*
* Description:  Mixes data to dsmMixBuffer.
*
* Input:        unsigned numElems       number of buffer elements to be mixed.
*                                       In mono modes an "element" is an
*                                       unsigned integer, and in stereo
*                                       two.
*
* Returns:      MIDAS error code. Mixed data is written to *dsmMixBuffer.
*
\****************************************************************************/

int CALLING dsmMixData(unsigned numElems);




/****************************************************************************\
*
* Function:     int dsmMix(unsigned channel, void *mixRoutine,
*                   unsigned volume, unsigned numElems);
*
* Description:  Mixes data for one channel. Used internally by dsmMixData().
*
* Input:        unsigned channel        channel number
*               void *mixRoutine        pointer to low-level mixing routine
*               unsigned volume         actual playing volume (volume in
*                                       channel structure is ignored)
*               unsigned numElems       number of elements to mix (see
*                                       dsmMixData())
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmMix(unsigned channel, void *mixRoutine, unsigned volume,
    unsigned numElems);




/****************************************************************************\
*
* Function:     int dsmClearBuffer(unsigned numElems)
*
* Description:  Clears the mixing buffer. Used only by dsmMixData().
*
* Input:        unsigned numElems       number of elements to clear
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int CALLING dsmClearBuffer(unsigned numElems);




#ifdef SUPPORTSTREAMS


/****************************************************************************\
*
* Function:     int dsmStartStream(unsigned channel, uchar *buffer,
*                   unsigned bufferLength, int sampleType);
*
* Description:  Starts playing a digital audio stream on a channel
*
* Input:        unsigned channel        channel number
*               uchar *buffer           pointer to stream buffer
*               unsigned bufferLength   buffer length in bytes
*               int sampleType          stream sample type
*               ulong rate              stream playing rate (in Hz)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmStartStream(unsigned channel, uchar *buffer, unsigned
    bufferLength, int sampleType, ulong rate);




/****************************************************************************\
*
* Function:     int dsmStopStream(unsigned channel);
*
* Description:  Stops playing digital audio stream on a channel
*
* Input:        unsigned channel        channel number
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmStopStream(unsigned channel);




/****************************************************************************\
*
* Function:     int dsmSetLoopCallback(unsigned channel,
*                   void (CALLING *callback)(unsigned channel));
*
* Description:  Sets sample looping callback to a channel
*
* Input:        unsigned channel        channel number
*               [..] *callback          pointer to callback function, NULL to
*                                       disable callback
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetLoopCallback(unsigned channel,
    void (CALLING *callback)(unsigned channel));




/****************************************************************************\
*
* Function:     int dsmSetStreamWritePosition(unsigned channel,
*                   unsigned position)
*
* Description:  Sets the stream write position on a channel
*
* Input:        unsigned channel        channel number
*               unsigned position       new stream write position
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING dsmSetStreamWritePosition(unsigned channel, unsigned position);



#endif /* #ifdef SUPPORTSTREAMS */


/* Prototypes for the actual mixing routines: */
void CALLING dsmMix8bitMonoMono(void);      /* 8-bit Mono => Mono */
void CALLING dsmMix8bitMonoStereo(void);    /* 8-bit Mono => Stereo */
void CALLING dsmMix8bitStereoMono(void);    /* 8-bit Stereo => Mono */
void CALLING dsmMix8bitStereoStereo(void);  /* 8-bit Stereo => Stereo */
void CALLING dsmMix16bitMonoMono(void);     /* 16-bit Mono => Mono */
void CALLING dsmMix16bitMonoStereo(void);   /* 16-bit Mono => Stereo */
void CALLING dsmMix16bitStereoMono(void);   /* 16-bit Stereo => Mono */
void CALLING dsmMix16bitStereoStereo(void); /* 16-bit Stereo => Stereo */







#ifdef __cplusplus
}
#endif



/****************************************************************************\
*       enum dsmFunctIDs
*       ----------------
* Description:  ID numbers for DSM functions
\****************************************************************************/

enum dsmFunctIDs
{
    ID_dsmInit = ID_dsm,
    ID_dsmClose,
    ID_dsmGetMixRate,
    ID_dsmOpenChannels,
    ID_dsmCalcVolTable,
    ID_dsmCloseChannels,
    ID_dsmClearChannels,
    ID_dsmMute,
    ID_dsmPause,
    ID_dsmSetMasterVolume,
    ID_dsmGetMasterVolume,
    ID_dsmSetAmplification,
    ID_dsmGetAmplification,
    ID_dsmPlaySound,
    ID_dsmReleaseSound,
    ID_dsmStopSound,
    ID_dsmSetRate,
    ID_dsmGetRate,
    ID_dsmSetVolume,
    ID_dsmGetVolume,
    ID_dsmSetSample,
    ID_dsmGetSample,
    ID_dsmChangeSample,
    ID_dsmSetPosition,
    ID_dsmGetPosition,
    ID_dsmGetDirection,
    ID_dsmSetPanning,
    ID_dsmGetPanning,
    ID_dsmMuteChannel,
    ID_dsmAddSample,
    ID_dsmRemoveSample,
    ID_dsmMixData,
    ID_dsmMix,
    ID_dsmMixMoNormal,
    ID_dsmMixStNormal,
    ID_dsmClearBuffer,
    ID_dsmStartStream,
    ID_dsmStopStream,
    ID_dsmSetLoopCallback,
    ID_dsmSetStreamWritePosition
};



#endif


/*
 * $Log: dsm.h,v $
 * Revision 1.7  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.6  1997/01/16 18:19:10  pekangas
 * Added support for setting the stream write position.
 * Stream data is no longer played past the write position
 *
 * Revision 1.5  1996/07/13 18:13:25  pekangas
 * Fixed to compile with Visual C
 *
 * Revision 1.4  1996/06/26 19:15:03  pekangas
 * Added sample loop callbacks
 *
 * Revision 1.3  1996/05/28 20:29:56  pekangas
 * Changed mixing routine prototypes and added new ones
 *
 * Revision 1.2  1996/05/26 20:55:20  pekangas
 * Added dsmStartStream and dsmStopStream
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/


//#include "mutils.h"
/*      MUTILS.H
 *
 * Miscellaneous utility functions for MIDAS Sound System used
 * by various system components.
 *
 * $Id: mutils.h,v 1.2 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/

#ifndef __MUTILS_H
#define __MUTILS_H


#ifdef __cplusplus
extern "C" {
#endif


/****************************************************************************\
*
* Function:     int mGetKey(void)
*
* Description:  Waits for a keypress and returns the read key
*
* Returns:      ASCII code for the key pressed. Extended keycodes are
*               returned with bit 8 set, eg. up arrow becomes \x148.
*
\****************************************************************************/

int CALLING mGetKey(void);



/****************************************************************************\
*
* Function:     int mStrLength(const char *str)
*
* Description:  Calculates the length of a ASCIIZ string
*
* Input:        char *str               pointer to string
*
* Returns:      String length excluding the terminating '\0'.
*
\****************************************************************************/

int CALLING mStrLength(const char *str);




/****************************************************************************\
*
* Function:     void mStrCopy(char *dest, const char *src);
*
* Description:  Copies an ASCIIZ string from *src to *dest.
*
* Input:        char *dest              pointer to destination string
*               char *src               pointer to source string
*
\****************************************************************************/

void CALLING mStrCopy(char *dest, const char *src);




/****************************************************************************\
*
* Function:     void mStrAppend(char *dest, const char *src);
*
* Description:  Appends an ASCIIZ string to the end of another.
*
* Input:        char *dest              pointer to destination string
*               char *src               pointer to source string
*
\****************************************************************************/

void CALLING mStrAppend(char *dest, const char *src);



/****************************************************************************\
*
* Function:     void mMemCopy(void *dest, const void *src, unsigned numBytes);
*
* Description:  Copies a memory block from *src to *dest.
*
* Input:        void *dest              pointer to destination
*               void *src               pointer to source
*               unsigned numBytes       number of bytes to copy
*
\****************************************************************************/

void CALLING mMemCopy(void *dest, const void *src, unsigned numBytes);




/****************************************************************************\
*
* Function:     int mMemEqual(const void *m1, const void *m2,
*                   unsigned numBytes);
*
* Description:  Compares two memory blocks.
*
* Input:        void *m1                pointer to memory block #1
*               void *m2                pointer to memory block #2
*               unsigned numBytes       number of bytes to compare
*
* Returns:      1 if the memory blocks are equal, 0 if not.
*
\****************************************************************************/

int CALLING mMemEqual(const void *m1, const void *m2, unsigned numBytes);




/****************************************************************************\
*
* Function:     long mHex2Long(const char *hex)
*
* Description:  Converts a hexadecimal string to a long integer.
*
* Input:        char *hex               pointer to hex string, ASCIIZ
*
* Returns:      Value of the string or -1 if conversion failure.
*
\****************************************************************************/

long CALLING mHex2Long(const char *hex);




/****************************************************************************\
*
* Function:     long mDec2Long(const char *dec)
*
* Description:  Converts an unsigned decimal string to a long integer
*
* Input:        char *dec               pointer to string, ASCIIZ
*
* Returns:      Value of the string or -1 if conversion failure.
*
\****************************************************************************/

long CALLING mDec2Long(const char *dec);




/***************************************************************************\
*
* Function:     char *mGetEnv(const char *envVar);
*
* Description:  Searches a string from the environment
*
* Input:        envVar                  environment variable name, ASCIIZ
*
* Returns:      Pointer to environment string value (ASCIIZ), NULL if string
*               was not found.
*
\***************************************************************************/

char * CALLING mGetEnv(const char *envVar);


#ifdef __cplusplus
}
#endif


#endif


/*
 * $Log: mutils.h,v $
 * Revision 1.2  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/



//#include "ems.h"
//#if defined(__WIN32__)||defined(__LINUX__)
//#include "mpoll.h"
//#endif
//#include "midasfx.h"
/*      midasfx.h
 *
 * MIDAS sound effect library
 *
 * $Id: midasfx.h,v 1.3 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __midasfx_h
#define __midasfx_h



/****************************************************************************\
*       enum fxChannels
*       ---------------
* Description:  Possible values for effect channel numbers apart from regular
*               channel numbers
\****************************************************************************/

enum fxChannels
{
    fxAutoChannel = 0xFFFF
};



/****************************************************************************\
*       struct fxChannel
*       ----------------
* Description:  Sound effect channel structure
\****************************************************************************/

typedef struct
{
    unsigned    sdChannel;              /* Sound Device channel number */
    unsigned    sampleHandle;           /* Handle for the sample that is
                                           being played on this channel */
    unsigned    playHandle;             /* Playing handle for the sound on
                                           this channel */
    int         priority;               /* Channel sound priority */
} fxChannel;



#ifdef __cplusplus
extern "C" {
#endif



/****************************************************************************\
*
* Function:     int fxInit(SoundDevice *SD)
*
* Description:  Initializes the sound effect library
*
* Input:        SoundDevice *SD         Pointer to the Sound Device that will
*                                       be used for playing the effects
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxInit(SoundDevice *SD);




/****************************************************************************\
*
* Function:     int fxClose(void)
*
* Description:  Uninitializes the sound effect library
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxClose(void);




/****************************************************************************\
*
* Function:     int fxLoadRawSample(char *fileName, unsigned sampleType,
*                   int loopSample, unsigned *sampleHandle)
*
* Description:  Loads a raw sample file into memory and adds the sample to
*               the sound device.
*
* Input:        char *fileName          sample file name
*               unsigned sampleType     sample type, see enum sdSampleType
*               int loopSample          1 if sample is looped, 0 if not
*               unsigned *sampleHandle  pointer to sample handle variable
*
* Returns:      MIDAS error code. The sample handle for the sample will be
*               written to *sampleHandle.
*
\****************************************************************************/

int CALLING fxLoadRawSample(char *fileName, unsigned sampleType,
    int loopSample, unsigned *sampleHandle);




/****************************************************************************\
*
* Function:     int fxFreeSample(unsigned sample)
*
* Description:  Deallocates a sample and frees it from the Sound Device.
*
* Input:        unsigned sample         sample handle for the sample to be
*                                       deallocated.
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxFreeSample(unsigned sample);




/****************************************************************************\
*
* Function:     int fxSetAutoChannels(int numChannels,
*                   unsigned *channelNumbers)
*
* Description:  Sets the channel numbers that can be used as automatic effect
*               channels.
*
* Input:        int numChannels             number of channels that can be
*                                           used
*               unsigned *channelNumbers    pointer to a table that contains
*                                           the channels that can be used
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxSetAutoChannels(int numChannels, unsigned *channelNumbers);




/****************************************************************************\
*
* Function:     int fxPlaySample(unsigned channel, unsigned sample,
*                   int priority, unsigned rate, unsigned volume, int panning,
*                   unsigned *playHandle)
*
* Description:  Starts playing a sound effect sample on a channel
*
* Input:        unsigned channel        channel number, or fxAutoChannel for
*                                       automatic selection
*               unsigned sample         sample handle
*               int priority            effect priority, the higher the value
*                                       the higher the priority
*               unsigned rate           effect initial sample rate
*               unsigned volume         effect initial volume (0-64)
*               int panning             effect initial panning, see enum
*                                       sdPanning
*               unsigned *playHandle    effect playing handle variable
*
* Returns:      MIDAS error code. The playing handle for the effect will be
*               written to *playHandle.
*
\****************************************************************************/

int CALLING fxPlaySample(unsigned channel, unsigned sample, int priority,
    unsigned rate, unsigned volume, int panning, unsigned *playHandle);




/****************************************************************************\
*
* Function:     int fxStopSample(unsigned playHandle)
*
* Description:  Stops playing a sample
*
* Input:        unsigned playHandle     sample playing handle (from
*                                       fxPlaySample)
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxStopSample(unsigned playHandle);



/****************************************************************************\
*
* Function:     int fxSetSampleRate(unsigned playHandle, ulong rate)
*
* Description:  Changes the sample rate for a sample that is being played
*
* Input:        unsigned playHandle     sample playing handle (from
*                                       fxPlaySample)
*               ulong rate              new rate
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxSetSampleRate(unsigned playHandle, ulong rate);




/****************************************************************************\
*
* Function:     int fxSetSampleVolume(unsigned playHandle, unsigned volume)
*
* Description:  Changes the volume for a sample that is being played
*
* Input:        unsigned playHandle     sample playing handle (from
*                                       fxPlaySample)
*               unsigned volume         new volume
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxSetSampleVolume(unsigned playHandle, unsigned volume);




/****************************************************************************\
*
* Function:     int fxSetSamplePanning(unsigned playHandle, int panning)
*
* Description:  Changes the panning position for a sample that is being played
*
* Input:        unsigned playHandle     sample playing handle (from
*                                       fxPlaySample)
*               int panning             new panning position
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxSetSamplePanning(unsigned playHandle, int panning);




/****************************************************************************\
*
* Function:     int fxSetSamplePriority(unsigned playHandle, int priority)
*
* Description:  Changes the priority for a sample that is being played
*
* Input:        unsigned playHandle     sample playing handle (from
*                                       fxPlaySample)
*               int priority            new playing priority
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING fxSetSamplePriority(unsigned playHandle, int priority);


#ifdef __cplusplus
}
#endif


#endif      /* #ifdef __midasfx_h */


/*
 * $Log: midasfx.h,v $
 * Revision 1.3  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.2  1996/09/28 06:39:22  jpaana
 * Converted from CR/LF and fixed some typos
 *
 * Revision 1.1  1996/09/22 23:17:48  pekangas
 * Initial revision
 *
*/


//#include "midasstr.h"
/*      midasstr.h
 *
 * MIDAS stream library
 *
 * $Id: midasstr.h,v 1.5 1997/01/16 19:31:53 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __midasstr_h
#define __midasstr_h


/* Hack for Linux: (FIXME) */
#ifdef __LINUX__
#include <pthread.h>
#endif

/****************************************************************************\
*       enum strStreamMode
*       ------------------
* Description:  Stream playing mode
\****************************************************************************/

enum strStreamMode
{
    strStreamPlayFile = 1,
    strStreamCallback,
    strStreamPoll
};


/****************************************************************************\
*       struct strStream
*       ----------------
* Description:  Stream playing structure
\****************************************************************************/

typedef struct _strStream
{
    unsigned    sdChannel;              /* channel number for the stream */
    uchar       *buffer;                /* pointer to stream buffer */
    unsigned    bufferSamples;          /* buffer length in samples */
    unsigned    bufferBytes;            /* buffer length in bytes */
    unsigned    bufferWritePos;         /* buffer write position */
    uchar       *fileBuffer;            /* file reading buffer */
    unsigned    fileBufferBytes;        /* file reading buffer size */
    unsigned    sampleSize;             /* size of one sample in bytes */
    int         streamMode;             /* see enum strStreamMode */
    fileHandle  f;                      /* stream file handle */
    ulong       fileLength;             /* stream file length in bytes */
    ulong       fileLeft;               /* number of bytes left in file */
    int         loop;                   /* 1 if the stream loops */
    volatile int threadExitFlag;        /* thread exit flag */
    unsigned    threadDelay;            /* delay between polling in thread */
    void (CALLING *callback)(uchar *buffer, struct _strStream *stream);
#ifdef __LINUX__
    pthread_t   playerThread;           /* player thread */
#endif
                                        /* stream playing callback */
    uchar       bufferClearVal;         /* the value that should be used to
                                           clear the buffer */
    uchar       filler[3];
} strStream;



#ifdef __cplusplus
extern "C" {
#endif



/****************************************************************************\
*
* Function:     int strInit(SoundDevice *SD)
*
* Description:  Initializes the stream library
*
* Input:        SoundDevice *SD         Pointer to the Sound Device that will
*                                       be used for playing the streams
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING strInit(SoundDevice *SD);




/****************************************************************************\
*
* Function:     int strClose(void)
*
* Description:  Uninitializes the stream library
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING strClose(void);



/****************************************************************************\
*
* Function:     int strPlayStreamFile(unsigned channel, char *fileName,
*                   unsigned sampleType, ulong sampleRate,
*                   unsigned bufferLength, int loop, strStream **stream)
*
* Description:  Starts playing a digital sound stream from a file. Creates a
*               new thread that will take care of reading the file and feeding
*               it to the stream buffer
*
* Input:        unsigned channel        channel number for the stream
*               char *fileName          stream file name
*               unsigned sampleType     stream sample type
*               ulong sampleRate        sampling rate
*               unsigned bufferLength   stream buffer length in milliseconds
*               int loop                1 if the stream should be looped,
*                                       0 if not
*               strStream **stream      pointer to stream state pointer
*
* Returns:      MIDAS error code. Pointer to the stream state structure will
*               be written to *stream
*
\****************************************************************************/

int CALLING strPlayStreamFile(unsigned channel, char *fileName,
    unsigned sampleType, ulong sampleRate, unsigned bufferLength, int loop,
    strStream **stream);




/****************************************************************************\
*
* Function:     int strPlayStreamPolling(unsigned channel,
*                   unsigned sampleType, ulong sampleRate,
*                   unsigned bufferLength, strStream **stream)
*
* Description:  Starts playing a stream in polling mode. Use
*               strFeedStreamData() to feed the stream data to the player
*
* Input:        unsigned channel        channel number for the stream
*               unsigned sampleType     stream sample type
*               ulong sampleRate        stream sampling rate
*               unsigned bufferLength   stream buffer length in milliseconds
*               strStream **stream      pointer to stream state pointer
*
* Returns:      MIDAS error code. Pointer to the stream state structure will
*               be written to *stream
*
\****************************************************************************/

int CALLING strPlayStreamPolling(unsigned channel, unsigned sampleType,
    ulong sampleRate, unsigned bufferLength, strStream **stream);



/****************************************************************************\
*
* Function:     int strPlayStreamCallback(unsigned sampleType,
*                  ulong sampleRate, unsigned bufferBytes,
*                  void (CALLING *callback)(uchar *buffer, strStream *stream))
*
* Description:  Starts playing a stream with a callback.
*
* Input:        unsigned sampleType     stream sample type
*               ulong sampleRate        stream sampling rate
*               unsigned bufferBytes    stream buffer size _IN BYTES_
*               ... *callback           stream player callback
*               strStream **stream      pointer to stream state pointer
*
* Returns:      MIDAS error code. Pointer to the stream state structure will
*               be written to *stream
*
* Notes:        The callback function will be called each time the whole
*               stream buffer needs to be filled. It receives as an argument
*               a pointer to the buffer, and the stream state pointer.
*
*               The function will be called from inside the mixing routine,
*               so it should return relatively rapidly - do not use this
*               function for, for example, loading data from disc.
*
\****************************************************************************/

int CALLING strPlayStreamCallback(unsigned sampleType, ulong sampleRate,
    unsigned bufferBytes,
    void (CALLING *callback)(uchar *buffer, strStream *stream));




/****************************************************************************\
*
* Function:     int strStopStream(strStream *stream)
*
* Description:  Stops playing a stream. This function will also destroy the
*               playback thread for stream file playback.
*
* Input:        strStream *stream       stream state pointer for the stream
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int CALLING strStopStream(strStream *stream);




/****************************************************************************\
*
* Function:     int strFeedStreamData(strStream *stream, uchar *data,
*                   unsigned numBytes, int feedAll, unsigned *numFed)
*
* Description:  Feeds sample data to a stream that is being played in polling
*               mode.
*
* Input:        strStream *stream       stream state pointer from
*                                       strPlayStreamPolling()
*               uchar *data             pointer to stream data
*               unsigned numBytes       number of bytes of data to feed. Note!
*                                       This must be a multiple of the stream
*                                       sample size
*               int feedAll             1 if all data should be fed in all
*                                       circumstances. The function will block
*                                       the current thread if this flag is 1
*                                       until all data is fed.
*               unsigned *numFed        pointer to a variable that will
*                                       contain the number of bytes actually
*                                       fed
*
* Returns:      MIDAS error code. The number of bytes of data actually fed is
*               written to *numFed.
*
\****************************************************************************/

int CALLING strFeedStreamData(strStream *stream, uchar *data,
    unsigned numBytes, int feedAll, unsigned *numFed);




/****************************************************************************\
*
* Function:     int strSetStreamRate(strStream *stream, ulong sampleRate)
*
* Description:  Changes the sampling rate for a stream
*
* Input:        strStream *stream       stream state pointer
*               ulong sampleRate        new sampling rate in Hz
*
* Returns:      MIDAS error code
*
\****************************************************************************/

int CALLING strSetStreamRate(strStream *stream, ulong sampleRate);




/****************************************************************************\
*
* Function:     int strSetStreamVolume(strStream *stream, unsigned volume)
*
* Description:  Changes the playback volume for a stream (the default is 64).
*
* Input:        strStream *stream       stream state pointer
*               unsigned volume         new volume (0-64)
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int CALLING strSetStreamVolume(strStream *stream, unsigned volume);




/****************************************************************************\
*
* Function:     int strSetStreamPanning(strStream *stream, int panning)
*
* Description:  Changes the panning for a stream (the default is middle).
*
* Input:        strStream *stream       stream state pointer
*               int panning             new panning position
*
* Returns:      MIDAS error code.
*
\****************************************************************************/

int CALLING strSetStreamPanning(strStream *stream, int panning);



/****************************************************************************\
*
* Function:     int strIsStreamFinished(strStream *stream, int *finished)
*
* Description:  Checks whether a given stream has reached the end of the
*               stream file or not. Only applies to streams played from a
*               file.
*
* Input:        strStream *stream       stream state pointer
*               int *finished           pointer to result variable
*
* Returns:      MIDAS error code. If the stream is finished, 1 will be written
*               to *finished, otherwise *finished will contain 0.
*
\****************************************************************************/

int CALLING strIsStreamFinished(strStream *stream, int *finished);




#ifdef __cplusplus
}
#endif


#endif      /* #ifdef __midasstr_h */


/*
 * $Log: midasstr.h,v $
 * Revision 1.5  1997/01/16 19:31:53  pekangas
 * Fixed to compile with Linux GCC (but do they work?)
 *
 * Revision 1.4  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.3  1997/01/16 18:25:08  pekangas
 * Implemented strSetStreamRate, strSetStreamVolume and strSetStreamPanning
 *
 * Revision 1.2  1996/09/28 06:39:46  jpaana
 * Converted from CR/LF and fixed some typos
 *
 * Revision 1.1  1996/09/22 23:17:48  pekangas
 * Initial revision
 *
*/



#ifdef __cplusplus
extern "C" {
#endif

/****************************************************************************\
*      MIDAS global variables:
\****************************************************************************/

#ifdef __WIN32__

extern SoundDevice GLOBALVAR WinWave;   /* Windows Wave Sound Device */
extern SoundDevice GLOBALVAR MixNoSound; /* Mixing NoSound SD */
#define NUMSDEVICES 2                   /* total number of Sound Devices */

#else /* __WIN32__ */

#ifdef __LINUX__

extern SoundDevice GLOBALVAR OSS;   /* OSS Sound Device */
#define NUMSDEVICES 1

#else /* __LINUX__ */

extern SoundDevice GLOBALVAR GUS;       /* Gravis UltraSound Sound Device */
extern SoundDevice GLOBALVAR GDC;       /* Gravis UltraSound Mixing Device */
extern SoundDevice GLOBALVAR PAS;       /* Pro Audio Spectrum Sound Device */
extern SoundDevice GLOBALVAR WSS;       /* Windows Sound System Sound Device*/
extern SoundDevice GLOBALVAR SB;        /* Sound Blaster Sound Device */
extern SoundDevice GLOBALVAR NoSound;   /* No Sound Sound Device */

#define NUMSDEVICES 6                   /* total number of Sound Devices */
#endif
#endif

extern SoundDevice GLOBALVAR *midasSD;  /* current Sound Device */

    /* array to pointers to all Sound Devices, in numbering and detection
       order - GUS is SD #1: */
extern SoundDevice * GLOBALVAR midasSoundDevices[NUMSDEVICES];

extern gmpPlayHandle GLOBALVAR midasPlayHandle;  /* Generic Module Player
                                                    playing handle */
extern int GLOBALVAR midasDisableEMS;   /* 1 if EMS usage is disabled
                                           (default 0) */
extern int GLOBALVAR midasSDNumber;     /* Sound Device number (-1 for
                                           autodetect, default -1) */
extern int GLOBALVAR midasSDPort;       /* Sound Device I/O port number
                                           (-1 for autodetect or SD default,
                                           default -1) */
extern int GLOBALVAR midasSDIRQ;        /* Sound Device IRQ number (-1 for
                                           autodetect or SD default,
                                           default -1) */
extern int GLOBALVAR midasSDDMA;        /* Sound Device DMA channel number
                                           (-1 for autodetect or SD default,
                                           default -1) */
extern int GLOBALVAR midasSDCard;       /* Sound Device sound card type
                                           (-1 for autodetect or SD default,
                                           default -1) */
extern unsigned GLOBALVAR midasMixRate; /* Sound Device mixing rate */
extern unsigned GLOBALVAR midasOutputMode; /* Sound Device output mode force
                                           bits, default 0 (SD default) */
extern int GLOBALVAR midasAmplification;  /* Forced amplification level or -1
                                             for SD default (default -1) */
extern int GLOBALVAR midasChannels;     /* number of channels open or 0 if no
                                           channels have been opened using
                                           midasOpenChannels() */
extern int GLOBALVAR midasPlayerNum;    /* timer music player number */


extern int GLOBALVAR midasEMSInit;      /* is EMS heap manager initialized? */
extern int GLOBALVAR midasTMRInit;      /* is TempoTimer initialized? */
extern int GLOBALVAR midasTMRPlay;      /* is sound beind player with timer?*/
extern int GLOBALVAR midasSDInit;       /* is Sound Device initialized? */
extern int GLOBALVAR midasSDChans;      /* are Sound Device channels open? */
extern int GLOBALVAR midasGMPInit;      /* is GMP initialized? */
extern int GLOBALVAR midasGMPPlay;      /* is GMP playing? */
extern int GLOBALVAR midasTMRMusic;     /* is music being player with timer?*/

extern void (CALLING *midasErrorExit)(char *msg);   /* error exit function */




/****************************************************************************\
*      MIDAS constants:
\****************************************************************************/



/****************************************************************************\
*      MIDAS global functions from MIDAS.C:
\****************************************************************************/


/****************************************************************************\
*
* Function:     void midasError(int errNum)
*
* Description:  Prints a MIDAS error message to stderr and exits to DOS
*
* Input:        int errNum              MIDAS error code
*
\****************************************************************************/

void CALLING midasError(int errNum);




/****************************************************************************\
*
* Function:     void midasUninitError(int errNum)
*
* Description:  Prints an error message to stderr and exits to DOS without
*               uninitializing MIDAS. This function should only be used
*               from midasClose();
*
* Input:        int errNum              MIDAS error code
*
\****************************************************************************/

void CALLING midasUninitError(int errNum);



/****************************************************************************\
*
* Function:     void midasDetectSD(void)
*
* Description:  Attempts to detect a Sound Device. Sets the global variable
*               midasSD to point to the detected Sound Device or NULL if no
*               Sound Device was detected
*
\****************************************************************************/

void CALLING midasDetectSD(void);



/****************************************************************************\
*
* Function:     void midasInit(void)
*
* Description:  Initializes MIDAS Sound System
*
\****************************************************************************/

void CALLING midasInit(void);



/****************************************************************************\
*
* Function:     void midasClose(void)
*
* Description:  Uninitializes MIDAS Sound System
*
\****************************************************************************/

void CALLING midasClose(void);



/****************************************************************************\
*
* Function:     void midasSetDefaults(void)
*
* Description:  Initializes MIDAS Sound System variables to their default
*               states. MUST be the first MIDAS function to be called.
*
\****************************************************************************/

void CALLING midasSetDefaults(void);




/****************************************************************************\
*
* Function:     void midasOpenChannels(int numChans);
*
* Description:  Opens Sound Device channels for sound and music output.
*
* Input:        int numChans            Number of channels to open
*
* Notes:        Channels opened with this function can be used for sound
*               playing, and modules played with midasPlayModule() will be
*               played through the last of these channels. This function is
*               provided so that the same number of channels can be open
*               the whole time throughout the execution of the program,
*               keeping the volume level constant. Note that you must ensure
*               that you open enough channels for all modules, otherwise
*               midasPlayModule() will fail.
*
\****************************************************************************/

void CALLING midasOpenChannels(int numChans);




/****************************************************************************\
*
* Function:     void midasCloseChannels(void);
*
* Description:  Closes Sound Device channels opened with midasOpenChannels().
*               Do NOT call this function unless you have opened the sound
*               channels used yourself with midasOpenChannels().
*
\****************************************************************************/

void CALLING midasCloseChannels(void);




/****************************************************************************\
*
* Function:     midasPlayModule(gmpModule *module, int numEffectChns)
*
* Description:  Starts playing a Generic Module Player module loaded to memory
*
* Input:        gmpModule *module       Pointer to loaded module structure
*               int numEffectChns       Number of channels to open for sound
*                                       effects. Ignored if sound channels
*                                       have already been opened with
*                                       midasOpenChannels().
*
* Returns:      Pointer to module structure. This function can not fail,
*               as it will call midasError() to handle all error cases.
*
* Notes:        The Sound Device channels available for sound effects are the
*               _first_ numEffectChns channels. So, for example, if you use
*               midasPlayModule(module, 3), you can use channels 0-2 for sound
*               effects. If you already have opened channels with
*               midasOpenChannels(), the module will be played with the last
*               possible channels, so that the first channels will be
*               available for sound effects. Note that if not enough channels
*               are open this function will fail.
*
\****************************************************************************/

void CALLING midasPlayModule(gmpModule *module, int numEffectChns);




/****************************************************************************\
*
* Function:     void midasStopModule(gmpModule *module)
*
* Input:        gmpModule *module       the module which is being played
*
* Description:  Stops playing a module and uninitializes the Module Player.
*               If sound channels were NOT opened through midasOpenChannels(),
*               but by letting midasPlayModule() open them, they will be
*               closed. Sound channels opened with midasOpenChannels() are NOT
*               closed and must be closed separately.
*
\****************************************************************************/

void CALLING midasStopModule(gmpModule *module);




/****************************************************************************\
*
* Function:     void midasSetErrorExit(void (CALLING *errorExit)(char *msg))
*
* Description:  Sets error exit function.
*
* Input:        void (CALLING *errorExit)() Pointer to the function that will
*                                           be called to exit the program with
*                                           an error message.
*
\****************************************************************************/

void midasSetErrorExit(void (CALLING *midasErrorExit)(char *msg));




#ifdef __cplusplus
}
#endif


#endif


/*
 * $Log: midas.h,v $
 * Revision 1.15  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.14  1996/10/14 14:57:24  pekangas
 * Changed version to 0.5.0
 *
 * Revision 1.13  1996/09/22 23:02:52  pekangas
 * Now #include midasfx.h and midasstr.h
 *
 * Revision 1.12  1996/09/21 17:18:01  jpaana
 * mpoll.h is included also in Linux
 *
 * Revision 1.11  1996/09/21 16:38:00  jpaana
 * Changed uss to oss
 *
 * Revision 1.10  1996/09/08 20:21:20  pekangas
 * Boosted version to RC2
 *
 * Revision 1.9  1996/08/06 20:35:51  pekangas
 * Now #includes mpoll.h for WIN32 targets
 *
 * Revision 1.8  1996/08/06 18:45:25  pekangas
 * Changed version number to 0.5.rc1
 *
 * Revision 1.7  1996/07/29 19:36:08  pekangas
 * Added MixNoSound Sound Device for Win32
 *
 * Revision 1.6  1996/07/13 20:32:19  pekangas
 * Fixed midasSoundDevices[] with Visual C
 *
 * Revision 1.5  1996/07/13 18:18:24  pekangas
 * Fixed to compile with Visual C
 *
 * Revision 1.4  1996/06/05 19:40:35  jpaana
 * Changed usswave to uss
 *
 * Revision 1.3  1996/05/25 15:49:57  jpaana
 * Small fix
 *
 * Revision 1.2  1996/05/24 16:20:36  jpaana
 * Added USSWave device and fixed Linux support
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/

/*****************************************************************************
*****************************************************************************/
/*****************************************************************************
*****************************************************************************/
/*****************************************************************************
*****************************************************************************/
/*****************************************************************************
*****************************************************************************/
/*****************************************************************************
*****************************************************************************/
//#include "mconfig.h"
/*      MCONFIG.C
 *
 * MIDAS Sound System configuration. Meant to be used with the simplified
 * MIDAS API, MIDAS.C
 *
 * $Id: mconfig.h,v 1.2 1997/01/16 18:41:59 pekangas Exp $
 *
 * Copyright 1996,1997 Housemarque Inc.
 *
 * This file is part of the MIDAS Sound System, and may only be
 * used, modified and distributed under the terms of the MIDAS
 * Sound System license, LICENSE.TXT. By continuing to use,
 * modify or distribute this file you indicate that you have
 * read the license and understand and accept it fully.
*/


#ifndef __MCONFIG_H
#define __MCONFIG_H


#ifdef __cplusplus
extern "C" {
#endif



/****************************************************************************\
*
* Function:     int midasConfig(void)
*
* Description:  MIDAS Sound System configuration. Prompts the user for all
*               configuration information and sets the MIDAS variables
*               accordingly. Call before midasInit() but after
*               midasSetDefaults().
*
* Returns:      1 if configuration was successful, 0 if not (Esc was pressed)
*
\****************************************************************************/

int CALLING midasConfig(void);




/****************************************************************************\
*
* Function:     void midasLoadConfig(char *fileName);
*
* Description:  Loads configuration from file saved using midasSaveConfig().
*
* Input:        char *fileName          configuration file name, ASCIIZ
*
\****************************************************************************/

void CALLING midasLoadConfig(char *fileName);




/****************************************************************************\
*
* Function:     void midasSaveConfig(char *fileName);
*
* Description:  Saves configuration to a file
*
* Input:        char *fileName          configuration file name, ASCIIZ
*
\****************************************************************************/

void CALLING midasSaveConfig(char *fileName);



#ifdef __cplusplus
}
#endif



#endif


/*
 * $Log: mconfig.h,v $
 * Revision 1.2  1997/01/16 18:41:59  pekangas
 * Changed copyright messages to Housemarque
 *
 * Revision 1.1  1996/05/22 20:49:33  pekangas
 * Initial revision
 *
*/

