
#include "midas.h"
#include "vga.h"

/* Some changes to demo behaviour for Assembly '96. The rules state that the
   demo must start without a configuration screen if either GUS or SB has been
   detected, and this does exactly that. In addition, if the user specifies
   _any_ command line arguments, the config will be run. Feel free to modify
   main(). To get the old behaviour, ie. a config screen always, comment out
   the next line: */
#define ASM96

/* Music playing position information - updated by UpdateInfo: */
unsigned        position;               /* Current position */
unsigned        pattern;                /* Current pattern number */
unsigned        row;                    /* Current row number */
int             syncInfo;               /* Music synchronization info */

/****************************************************************************\
*
* Function:     void Error(char *msg)
*
* Description:  Displays an error message and exits to DOS
*
* Input:        char *msg               error message
*
\****************************************************************************/

void Error(char *msg)
{
    vgaSetMode(3);
    printf("Error: %s\n", msg);
#ifndef NOMUSIC
    midasClose();           /* IMPORTANT! */
#endif
    exit(EXIT_FAILURE);
}

/****************************************************************************\
*
* Function:     void Errorcode(int errorcode)
*
* Description:  Displays an error message related to a MIDAS error code and
*               exits to DOS
*
* Input:        int errorcode           MIDAS error code
*
\****************************************************************************/

void Errorcode(int errorcode)
{
    Error(errorMsg[errorcode]);
}

/****************************************************************************\
*
* Function:     void WaitFrame(void)
*
* Description:  Waits for the next frame
*
\****************************************************************************/

void WaitFrame(void)
{
    unsigned    old;

#ifdef NOMUSIC
    vgaWaitNoVR();
    vgaWaitVR();
#else

    /* Playing music - wait for frame counter to change: */
    old = frameCount;
    while ( old == frameCount );
#endif
}




/****************************************************************************\
*
* Function:     void UpdateInfo(void)
*
* Description:  Updates song playing information (defined at the beginning of
*               this file)
*
* Note:         To use the position information from Assembler, do the
*               following (almost - get the idea?)
*
*               INCLUDE "midas.inc"
*               ...
*               GLOBAL  UpdateInfo : LANG
*               ...
*               EXTRN   position : dword
*               EXTRN   pattern : dword
*               EXTRN   row : dword
*               EXTRN   syncInfo : dword
*               ...
*               CODESEG
*               ...
*               PROC    SuperRoutine    NEAR
*               ...
*               call    UpdateInfo C
*               cmp     [position],17
*               je      @@design
*               ...
*
\****************************************************************************/

void CALLING UpdateInfo(void)
{
    static gmpInformation   *info;
    int         error;

    /* Get GMP playing information: (this can't fail but let's just play it
       safe) */
    if ( (error = gmpGetInformation(midasPlayHandle, &info)) != OK )
        midasError(error);

    /* Store interesting information in easy-to-access variables: */
    position = info->position;
    pattern = info->pattern;
    row = info->row;
    syncInfo = info->syncInfo;
}



/****************************************************************************\
*
* Function:     void SyncCallback(unsigned syncNum, unsigned position,
*                   unsigned row);
*
* Description:  Music synchronization callback function. Called by GMPlayer
*               whenever command 'W' is encountered (XMs and S3Ms).
*
* Input:        unsigned syncNum        synchronization command infobyte
*               unsigned position       current position
*               unsigned row            current row
*
* Notes:        This function is called from inside the music player timer
*               interrupt! Therefore it may not take very much time (more
*               than a few rasterlines maximum) and shouldn't do really
*               much anything...
*
\****************************************************************************/

void CALLING SyncCallback(unsigned syncNum, unsigned position, unsigned row)
{
    static      border = 0;

    /* Prevent warnings: */
    position = position;
    row = row;

    /* Check if the infobyte is interesting - do something only when command
       "W42" is encountered: */
    if ( syncNum == 0x42 )
    {
        /* Yeah, yeah, flash the border! */
        border ^= 1;
        vgaSetBorder(border);
    }
}




