#include <tricks.h>
#include <xreader.h>
#include <fstream.h>

// GLOBALS ************************************************************

static byte buf[64000];
static obj *O;
static obj *rot_O;
static byte sp_envmap[65536];
static byte pal[768];
static int ant;
static int *df;

#define MAX_X 318
#define MAX_Y 198
#define MIN_X 2
#define MIN_Y 2
	
struct pos {
		float xp[200];
		float yp[200];
		float zp[200];
		float xv[200];
		float yv[200];
		float zv[200];
};

pos keyframe;

struct drawpoly {
	vertex points[10];
	int pcount;
};

// PROTOTYPES *******************************************************

void drawObj(int*list,int num,byte *dest);

void init(char *xfile, char* mapname, char *splinefile);
void loop();
void deinit();

int readframe(char *);
void plotfustrum(int,int,int,int);
void clip(drawpoly *,drawpoly *);
void clipLeft(drawpoly *,vertex,vertex);
void clipRight(drawpoly *,vertex,vertex);
void clipTop(drawpoly *,vertex,vertex);
void clipBottom(drawpoly *,vertex,vertex);
void mapPoly(drawpoly *);
void fillScanLine(float,float,float,float,float,float,byte*);
int clipchk(vertex *);
void drawPoly( int, byte * );		 
void rotate(const obj *,obj *,float,float,float,float,float,float,float,float);
float spline(float,float*);


// CODE *****************************************************************


int comp2(const void *a,const void *b)
{
   float za = rot_O->f[*(int*)a].z;
   float zb = rot_O->f[*(int*)b].z;

   if(za>zb)       
      return -1;     
   else if(za<zb)
      return 1;
   else
      return 0;
}

inline void drawTxtline(int A,int B,int uA,int vA,int uB,int vB,byte *b)
{
  if(A==B) return;

  int width,du,dv;

  if(A>B){
    swap(A,B);
    swap(uA,uB);
    swap(vA,vB);
  }

  b+=A;
  width=B-A;
  du=(uB-uA)/width;
  dv=(vB-vA)/width;
  *b=sp_envmap[(uA>>8)+(vA&0xff00)];

  while(width--){
      uA+=du;
      vA+=dv;
      *++b=sp_envmap[(uA>>8)+(vA&0xff00)];
  };
}


void loop()
{
		float xpos=0,ypos=0,zpos=-10,xvin=0,yvin=0,zvin=0;
 for(float t=1;t<(ant-3);t+=0.02) {
    xpos=spline(t,keyframe.xp);
    ypos=spline(t,keyframe.yp);
    zpos=spline(t,keyframe.zp);
    xvin=spline(t,keyframe.xv);
    yvin=spline(t,keyframe.yv);
    zvin=spline(t,keyframe.zv);
		
    rotate(O,rot_O,xvin,yvin,zvin,xpos,ypos,zpos,2,600);
    int count=0;
    for(int i=0;i<rot_O->faces;i++) {
      if(rot_O->f[i].zn>0)
         df[count++] = i;
    }
    qsort(df,count,sizeof(int),comp2);
    drawObj(df,count,buf);
    plotfustrum(1,2,318,199);
    fastMemCopy(buf,vga,16000);
    fastMemClear(buf,16000);

    if(kbhit())
      return;
  }
}

int readframe(char *filename) {
		char tmp;
		int ant;
		ifstream fin(filename);
		fin >> ant >> tmp;
		for(int a=0;a<ant;a++){
			fin >> keyframe.xp[a] >> tmp >> keyframe.yp[a] >> tmp >> keyframe.zp[a] >> tmp;
			fin >> keyframe.xv[a] >> tmp >> keyframe.yv[a] >> tmp >> keyframe.zv[a];
		}	
		fin.close();
	return ant;
}

void plotfustrum(int x1,int y1,int x2,int y2) {
	for(int i=x1;i<x2;i++) {
		buf[i+y1*320]=0;
		buf[i+y2*320]=0;
	}
	for(i=y1;i<y2;i++) {
		buf[i*320+x1]=0;
		buf[i*320+x2]=0;
	}
}

float spline(float t,float *coord){
  long tid;
  float a,b,c,d,y0,y1,yd0,yd1,y;
  tid=t;
  y0=coord[tid+1];
  y1=coord[tid+2];
  yd0=(coord[tid+2]-coord[tid+0])/2;
  yd1=(coord[tid+3]-coord[tid+1])/2;
  d=y0;
  c=yd0;
  a=yd1-2*y1+c+2*d;
  b=(yd1-3*a-c)/2;
  t=t-tid;
  y=a*t*t*t + b*t*t + c*t + d;
  return y;
}

#define t(x2,x1,LIMIT) ((LIMIT-x1)/(x2-x1));

void clipLeft(drawpoly *Polygon,vertex V1,vertex V2) {
	float dx,dy,m=1;
  m=t(V2.x,V1.x,MIN_X);
	// Bda inne
	if ( (V1.x>=MIN_X) && (V2.x>=MIN_X))
		Polygon->points[Polygon->pcount++]=V2;
	// P vg ut
  if((V1.x>=MIN_X) && (V2.x<MIN_X)) {
    Polygon->points[Polygon->pcount].x=MIN_X;
    Polygon->points[Polygon->pcount].u=V1.u+m*(V2.u-V1.u);
    Polygon->points[Polygon->pcount].v=V1.v+m*(V2.v-V1.v);
    Polygon->points[Polygon->pcount++].y=V1.y+m*(V2.y-V1.y);
  }
	// P vg in
  if((V1.x<MIN_X) && (V2.x>=MIN_X)) {
	Polygon->points[Polygon->pcount].x=MIN_X;
    Polygon->points[Polygon->pcount].u=V1.u+m*(V2.u-V1.u);
    Polygon->points[Polygon->pcount].v=V1.v+m*(V2.v-V1.v);
    Polygon->points[Polygon->pcount++].y=V1.y+m*(V2.y-V1.y);
    Polygon->points[Polygon->pcount++]=V2;
  }
}

void clipRight(drawpoly *Polygon,vertex V1,vertex V2) {
	float dx,dy,m=1;
  m=t(V2.x,V1.x,MAX_X);
	if((V1.x<=MAX_X) && (V2.x<=MAX_X))
		Polygon->points[Polygon->pcount++]=V2;

  if((V1.x<=MAX_X) && (V2.x>MAX_X)) {
    Polygon->points[Polygon->pcount].x=MAX_X;
    Polygon->points[Polygon->pcount].u=V1.u+m*(V2.u-V1.u);
    Polygon->points[Polygon->pcount].v=V1.v+m*(V2.v-V1.v);
    Polygon->points[Polygon->pcount++].y=V1.y+m*(V2.y-V1.y);
  }
  if((V1.x>MAX_X) && (V2.x<=MAX_X)) {
    Polygon->points[Polygon->pcount].x=MAX_X;
    Polygon->points[Polygon->pcount].u=V1.u+m*(V2.u-V1.u);
    Polygon->points[Polygon->pcount].v=V1.v+m*(V2.v-V1.v);
    Polygon->points[Polygon->pcount++].y=V1.y+m*(V2.y-V1.y);
    Polygon->points[Polygon->pcount++]=V2;
  }
}

void clipTop(drawpoly *Polygon,vertex V1,vertex V2) {
	float dx,dy,m=1;
  m=t(V2.y,V1.y,MIN_Y);

	if((V1.y>=MIN_Y) && (V2.y>=MIN_Y))
     Polygon->points[Polygon->pcount++]=V2;

  if((V1.y>=MIN_Y) && (V2.y<MIN_Y)) {
	  Polygon->points[Polygon->pcount].x=V1.x+(V2.x-V1.x)*m;
	  Polygon->points[Polygon->pcount].u=V1.u+(V2.u-V1.u)*m;
	  Polygon->points[Polygon->pcount].v=V1.v+(V2.v-V1.v)*m;
	  Polygon->points[Polygon->pcount++].y=MIN_Y;
  }

  if((V1.y<MIN_Y) && (V2.y>=MIN_Y)) {
		Polygon->points[Polygon->pcount].x=V1.x+(V2.x-V1.x)*m;
	  Polygon->points[Polygon->pcount].u=V1.u+(V2.u-V1.u)*m;
	  Polygon->points[Polygon->pcount].v=V1.v+(V2.v-V1.v)*m;
    Polygon->points[Polygon->pcount++].y=MIN_Y;
    Polygon->points[Polygon->pcount++]=V2;
  }
}

void clipBottom(drawpoly *Polygon,vertex V1,vertex V2) {
  float dx,dy, m=1;
  m=t(V2.y,V1.y,MAX_Y);

	if((V1.y<=MAX_Y) && (V2.y<=MAX_Y))
		Polygon->points[Polygon->pcount++]=V2;

	if((V1.y<=MAX_Y) && (V2.y>MAX_Y)) {
		Polygon->points[Polygon->pcount].x=V1.x+(V2.x-V1.x)*m;
	  Polygon->points[Polygon->pcount].u=V1.u+(V2.u-V1.u)*m;
	  Polygon->points[Polygon->pcount].v=V1.v+(V2.v-V1.v)*m;
    Polygon->points[Polygon->pcount++].y=MAX_Y;
   }

	if((V1.y>MAX_Y) && (V2.y<=MAX_Y)) {
    Polygon->points[Polygon->pcount].x=V1.x+(V2.x-V1.x)*m;
	  Polygon->points[Polygon->pcount].u=V1.u+(V2.u-V1.u)*m;
	  Polygon->points[Polygon->pcount].v=V1.v+(V2.v-V1.v)*m;
    Polygon->points[Polygon->pcount++].y=MAX_Y;
    Polygon->points[Polygon->pcount++]=V2;
  }
}

drawpoly TmpPoly;

void clip(drawpoly *Polygon,drawpoly *Clipped) {
	int v,d,chk=0;
	
  Clipped->pcount=0;
  TmpPoly.pcount=0;

	for (v=0;v<(Polygon->pcount);v++) {
		d=v+1;
		if(d==Polygon->pcount) d=0;
		clipLeft(&TmpPoly,Polygon->points[v],Polygon->points[d]);
  }
	for (v=0; v<TmpPoly.pcount;v++) {
		d=v+1;
    if(d==TmpPoly.pcount) d=0;
			clipRight(Clipped,TmpPoly.points[v],TmpPoly.points[d]);
  }
  TmpPoly.pcount=0;
  for (v=0; v<Clipped->pcount;v++) {
		d=v+1;
    if(d==Clipped->pcount) d=0;
			clipTop(&TmpPoly,Clipped->points[v],Clipped->points[d]);
  }
  Clipped->pcount=0;
  for (v=0; v<TmpPoly.pcount;v++) {
		d=v+1;
    if(d==TmpPoly.pcount) d=0;
			clipBottom(Clipped,TmpPoly.points[v],TmpPoly.points[d]);
  }
}

void drawObj(int *faces,int num,byte *dest){
	drawpoly dp;
	drawpoly clipped;
	dp.pcount=3;
	for(int i=0;i<num;i++,faces++) {
	
		int p1=rot_O->f[*faces].p1;
		int p2=rot_O->f[*faces].p2;
		int p3=rot_O->f[*faces].p3;
			
		dp.points[0].x=(rot_O->v[ p1 ].px>>8);
		dp.points[0].y=rot_O->v[ p1 ].py;
		dp.points[0].z=rot_O->v[ p1 ].z;			
		dp.points[0].u=(rot_O->v[ p1 ].xn*120)+127;
		dp.points[0].v=(rot_O->v[ p1 ].yn*120)+127;

		dp.points[1].x=(rot_O->v[ p2 ].px>>8);
		dp.points[1].y=rot_O->v[ p2 ].py;
		dp.points[1].z=rot_O->v[ p2 ].z;			
		dp.points[1].u=(rot_O->v[ p2 ].xn*120)+127;
		dp.points[1].v=(rot_O->v[ p2 ].yn*120)+127;

		dp.points[2].x=(rot_O->v[ p3 ].px>>8);
		dp.points[2].y=rot_O->v[ p3 ].py;
		dp.points[2].z=rot_O->v[ p3 ].z;			
		dp.points[2].u=(rot_O->v[ p3 ].xn*120)+127;
		dp.points[2].v=(rot_O->v[ p3 ].yn*120)+127;
		switch(clipchk(dp.points)){
			case 0:
				drawPoly( *faces, dest );
				break;
			case 1:
			case 2:
				clip(&dp,&clipped);
				mapPoly(&clipped);
				break;
			default: break;
			}
	 	}
	faces-=num;
}

int clipchk(vertex *v) {
	int cnt=0;	
	for(int i=0;i<3;i++) {
		if((v[i].x<MIN_X) || (v[i].x>MAX_X) || (v[i].y<MIN_Y) || (v[i].y>MAX_Y))
			cnt++;
	}
	return cnt;
}

void mapPoly(drawpoly *coords){

  int i,top=0,a,b;
  
  float y,x1,x2,u1,u2,v1,v2,
        dx_a,dx_b,du_a,du_b,dv_a,dv_b;
  int height_a,height_b;

	for(i=1; i<coords->pcount; i++){
    if(coords->points[i].y < coords->points[top].y)
      top = i;
  }
  a=top+1;
  b=top-1;
  if(a>(coords->pcount-1)) a=0;
  if(b<0) b=coords->pcount-1;

  y =coords->points[top].y;
  x1=coords->points[top].x;
  x2=x1;
  u1=coords->points[top].u;
  u2=u1;
  v1=coords->points[top].v;
  v2=v1;

  height_a=coords->points[a].y-coords->points[top].y+1;
  height_b=coords->points[b].y-coords->points[top].y+1;

  if(height_a) {
    dx_a=(coords->points[a].x-coords->points[top].x)/height_a;
    du_a=(coords->points[a].u-coords->points[top].u)/height_a;
    dv_a=(coords->points[a].v-coords->points[top].v)/height_a;
  }
  if(height_b) {
    dx_b=(coords->points[b].x-coords->points[top].x)/height_b;
    du_b=(coords->points[b].u-coords->points[top].u)/height_b;
    dv_b=(coords->points[b].v-coords->points[top].v)/height_b;
  }
  int Y=y;

  for(i=coords->pcount-1;i>0;){
    while(height_a && height_b){

      if(x1<x2)
        fillScanLine(x1-1,x2,u1,u2,v1,v2,buf+QuickFist(Y*320+x1-1));
      else
        fillScanLine(x2-1,x1,u2,u1,v2,v1,buf+QuickFist(Y*320+x2-1));
      Y++;                                   
      height_a--;
      height_b--;
      x1+=dx_a;
      x2+=dx_b;
      u1+=du_a;
      u2+=du_b;
      v1+=dv_a;
      v2+=dv_b;
    }


    if(!height_a){
      int na;
      na=a+1;
      if(na>(coords->pcount-1)) na=0;
      height_a=coords->points[na].y-coords->points[a].y+1;
      if(height_a){
        dx_a=(coords->points[na].x-coords->points[a].x)/height_a;
        du_a=(coords->points[na].u-coords->points[a].u)/height_a;
        dv_a=(coords->points[na].v-coords->points[a].v)/height_a;
			}
			x1=coords->points[a].x;
      u1=coords->points[a].u;
      v1=coords->points[a].v;
      i--;
      a=na;
    }else
    if(!height_b){
      int nb;
      nb=b-1;
      if(nb<0) nb=coords->pcount-1;
      height_b=coords->points[nb].y-coords->points[b].y+1;
      if(height_b){
        dx_b=(coords->points[nb].x-coords->points[b].x)/height_b;
        du_b=(coords->points[nb].u-coords->points[b].u)/height_b;
        dv_b=(coords->points[nb].v-coords->points[b].v)/height_b;
			}
 			x2=coords->points[b].x;
      u2=coords->points[b].u;
      v2=coords->points[b].v;
      i--;
      b=nb;
    } 

  }   
}

void fillScanLine(float x1,float x2,float u1,float u2,float v1,float v2,byte *b) {
    
  float width,du,dv;
  int u,v;

  width=x2-x1;
  if(width){                              
    du=(u2-u1)/width;
    dv=(v2-v1)/width;
  }
  for(int x=int(x1);x<int(x2);x++) {
    *b++=sp_envmap[(int(v1)<<8)+int(u1)];

    u1+=du;
    v1+=dv;
  }
} 

void drawPoly( int face, byte *dest )		 
{
	 int p1=rot_O->f[face].p1;
   int p2=rot_O->f[face].p2;
   int p3=rot_O->f[face].p3;

      int p1y=rot_O->v[p1].py;
      int p2y=rot_O->v[p2].py;
      int p3y=rot_O->v[p3].py;

      if(p1y>p2y) {
         swap(p1y,p2y);
         swap(p1 ,p2);
      }

      if(p1y>p3y) {
         swap(p1y,p3y);
         swap(p1 ,p3);
      }

      int dyA=p2y-p1y;
      int dyB=p3y-p1y;

      int p1x=rot_O->v[p1].px;       // redan shiftade 8 tecken upp!
      int p2x=rot_O->v[p2].px;
      int p3x=rot_O->v[p3].px;

      int p1u=(QuickFist(rot_O->v[p1].xn*120)+127)<<8;
      int p1v=(QuickFist(rot_O->v[p1].yn*120)+127)<<8;
      int p2u=(QuickFist(rot_O->v[p2].xn*120)+127)<<8;
      int p2v=(QuickFist(rot_O->v[p2].yn*120)+127)<<8;
      int p3u=(QuickFist(rot_O->v[p3].xn*120)+127)<<8;
      int p3v=(QuickFist(rot_O->v[p3].yn*120)+127)<<8;

      int A =p1x;
      int B =p1x;
      int uA=p1u;
      int uB=p1u;
      int vA=p1v;
      int vB=p1v;

      int dxA,duA,dvA;
      int dxB,duB,dvB;

      if(dyA){
         dxA=(p2x-p1x)/dyA;
         duA=(p2u-p1u)/dyA;
         dvA=(p2v-p1v)/dyA;
      }else{
         dxA=(p2x-p1x);
         duA=(p2u-p1u);
         dvA=(p2v-p1v);
         dyA++;
      }

      if(dyB){
         dxB=(p3x-p1x)/dyB;
         duB=(p3u-p1u)/dyB;
         dvB=(p3v-p1v)/dyB;
      }else{
         dxB=(p3x-p1x);
         duB=(p3u-p1u);
         dvB=(p3v-p1v);
         dyB++;
      }

      byte *b=dest+p1y*320;

      do{
         drawTxtline(A>>8,B>>8,uA,vA&0xff00,uB,vB&0xff00,b);
         dyA--;
         dyB--;
         if(dyA<0&&dyB<0) break;

         A +=dxA;
         B +=dxB;
         uA+=duA;
         vA+=dvA;
         uB+=duB;
         vB+=dvB;

         if(!dyA){
            dyA=p3y-p2y;
            if(dyA){
               dxA=(p3x-p2x)/dyA;
               duA=(p3u-p2u)/dyA;
               dvA=(p3v-p2v)/dyA;
            }else{
               dxA=(p3x-p2x);
               duA=(p3u-p2u);
               dvA=(p3v-p2v);
            }
         }else if(!dyB){
            dyB=p2y-p3y;
            if(dyB){
               dxB=(p2x-p3x)/dyB;
               duB=(p2u-p3u)/dyB;
               dvB=(p2v-p3v)/dyB;
            }else{
               dxB=(p2x-p3x);
               duB=(p2u-p3u);
               dvB=(p2v-p3v);
            }
         }
         b+=320;
      }while(dyA>=0&&dyB>=0);
}

void rotate(const obj *O,obj *rot_obj,float x, float y, float z,float xpos,float ypos,float zpos,float zd, float p )
{
   // precalc values..
	 float tempx,tempy,tempz;
   float sx = sin(x);
   float sy = sin(y);
   float sz = sin(z);

   float cx = cos(x);
   float cy = cos(y);
   float cz = cos(z);

   float mx1 = sx*sy*sz + cz*cx;
   float my1 = sx*cy;
   float mz1 = sz*cx - sx*sy*cz;

   float mx2 = cx*sy*sz - cz*sx;
   float my2 = cx*cy;
   float mz2 = -cz*sy*cx - sz*sx;

   float mx3 = -sz*cy;
   float my3 = sy;
   float mz3 = cz*cy;

   for( int i=0; i<O->vertices; i++ )
   {
      // rotate coords

      tempx = O->v[i].x;
      tempy = O->v[i].y;
      tempz = O->v[i].z;

      float tx,ty,tz;

      tx = tempx*mx1 + tempy*my1 + tempz*mz1;
      ty = tempx*mx2 + tempy*my2 + tempz*mz2;
      tz = tempx*mx3 + tempy*my3 + tempz*mz3;
			rot_O->v[i].x=tx;
			rot_O->v[i].y=ty;
			rot_O->v[i].z=tz;
      // project

      rot_O->v[i].px = (QuickFist(((xpos+tx)/(zpos+tz))*p) + 160)<<8;
      rot_O->v[i].py = QuickFist(((ypos+ty)/(zpos+tz))*p*.8) + 100;

      // rotate normals

			tx = O->v[i].xn;
      ty = O->v[i].yn;
      tz = O->v[i].zn;

      rot_O->v[i].xn = tx*mx1 + ty*my1 + tz*mz1;
      rot_O->v[i].yn = tx*mx2 + ty*my2 + tz*mz2;
      rot_O->v[i].zn = tx*mx3 + ty*my3 + tz*mz3;
   }

   for( i=0; i<O->faces; i++) {
      float p1x = rot_O->v[O->f[i].p1].px;
      float p1y = rot_O->v[O->f[i].p1].py;
      float p1z = rot_O->v[O->f[i].p1].z;

      float p2x = rot_O->v[O->f[i].p2].px;
      float p2y = rot_O->v[O->f[i].p2].py;
      float p2z = rot_O->v[O->f[i].p2].z;

      float p3x = rot_O->v[O->f[i].p3].px;
      float p3y = rot_O->v[O->f[i].p3].py;
      float p3z = rot_O->v[O->f[i].p3].z;
      rot_O->f[i].zn = ((p3y - p2y) * (p1x - p2x))
										 - ((p1y - p2y) * (p3x - p2x));

      rot_O->f[i].z  = (p1z+p2z+p3z);
   }
}



void init(char *xfile, char* mapname, char *splinefile)
{
   O=new obj;
   rot_O=new obj;
   readobj(O,xfile);
   readobj(rot_O,xfile);
   loadmap(mapname,sp_envmap,pal);
   df=new int[rot_O->faces];  // faces to be drawn..
   ant=readframe(splinefile);

   for(int i=0;i<256;i++)
      Pal(i,pal[i*3],pal[i*3+1],pal[i*3+2]);
}

void deinit()
{
   delete[] O->v;
   delete[] O->f;
   delete O;
   delete[] rot_O->v;
   delete[] rot_O->f;
   delete rot_O;
   delete[] df;
}
