#include <tricks.h>
#include <xreader.h>

// GLOBALS ************************************************************

#define SCRWID 320
#define ZMUL 900000

static int  wb_zbuf[256000];
static byte wb_pal[768];
static byte wb_texture[65536];
static short wb_hole[256000];

//static int  *df;
//static obj  *O;
static int  wb_offset;

// PROTOTYPES *******************************************************

void wb_drawObj(byte *dest);
void wb_init();
void wb_precalc();
void wb_deinit();
void wb_drawhole();
void wb_loop();
inline void wb_drawTxtZ(int &xA,int &xB,int uA,int vA,int zA,
                       int &uB,int &vB,int &zB,byte *b,int *z);
int comp(const void *a,const void *b);


// CODE *****************************************************************

void wb_loop()
{
  static float sinus = sin(-0.05);
  static float cosin = cos(-0.05);
  static float posx=0;
  static float posy=100;

  float tmp = posx*cosin + posy*sinus;
  posy = posy*cosin - posx*sinus;
  posx = tmp;
  wb_offset=QuickFist(posy+100)*640+QuickFist(posx+160);

  rotate(O,0.01,0.01,0.02,2.5,600);
  wb_drawhole();
  wb_drawObj(buf);
}  

int comp(const void *a,const void *b)
{
   float za = O->f[*(int*)a].z;
   float zb = O->f[*(int*)b].z;

   if(za>zb)       
      return -1;     
   else if(za<zb)
      return 1;
   else
      return 0;
}

inline void wb_drawTxtZ(int &xA,int &xB,int uA,int vA,int zA,
                       int &uB,int &vB,int &zB,byte *b,int *z)
{
  int width,du,dv,dz;

  width=xB-xA;
  du=(uB-uA)/width;
  dv=(vB-vA)/width;
  dz=(zB-zA)/width;

  do{
    if(zA<*z){
      *b=wb_texture[(uA>>8)+(vA&0xff00)];
    }
    b++;
    z++;
    uA+=du;
    vA+=dv;
    zA+=dz;
  }while(width--);
}


void wb_drawObj(byte *dest)
{
  int count=0;              // number of faces..

  for(int i=0;i<O->faces;i++)
    if(O->f[i].zn>0)
      df[count++]=i;

  qsort(df,count,sizeof(int),comp);
  i=count;
  do{
    int p1=O->f[*df].p1;
    int p2=O->f[*df].p2;
    int p3=O->f[*df].p3;

    int p1y=O->v[p1].py;
    int p2y=O->v[p2].py;
    int p3y=O->v[p3].py;

    if(p1y>p2y) {
      swap(p1y,p2y);
      swap(p1 ,p2);
    }

    if(p1y>p3y) {
      swap(p1y,p3y);
      swap(p1 ,p3);
    }

    int dyA=p2y-p1y;
    int dyB=p3y-p1y;

    int xA,xB;
    int uA,uB;
    int vA,vB;
    int zA,zB;

    xA=xB  =O->v[p1].px;                         // px<<8 done..
    uA=uB  =(QuickFist(O->v[p1].xn*30720)+32512)&0xefffffff;  // 32512 = 127<<8;
    vA=vB  =(QuickFist(O->v[p1].zn*30720)+32512)&0xefffffff;  // 30720 = 120<<8;
    zA=zB  =QuickFist(O->v[p1].z*ZMUL);

    int p2x=O->v[p2].px;
    int p2u=(QuickFist(O->v[p2].xn*30720)+32512)&0xefffffff;
    int p2v=(QuickFist(O->v[p2].zn*30720)+32512)&0xefffffff;
    int p2z=QuickFist(O->v[p2].z*ZMUL);

    int p3x=O->v[p3].px;
    int p3u=(QuickFist(O->v[p3].xn*30720)+32512)&0xefffffff;
    int p3v=(QuickFist(O->v[p3].zn*30720)+32512)&0xefffffff;
    int p3z=QuickFist(O->v[p3].z*ZMUL);

    int dxA,duA,dvA,dzA;
    int dxB,duB,dvB,dzB;

    if(dyA){
      dxA=(p2x-xA)/dyA;
      duA=(p2u-uA)/dyA;
      dvA=(p2v-vA)/dyA;
      dzA=(p2z-zA)/dyA;
    }else{
      dyA++;
      dxA=(p2x-xA);
      duA=(p2u-uA);
      dvA=(p2v-vA);
      dzA=(p2z-zA);
    }

    if(dyB){
      dxB=(p3x-xA)/dyB;
      duB=(p3u-uA)/dyB;
      dvB=(p3v-vA)/dyB;
      dzB=(p3z-zA)/dyB;
    }else{
      dyB++;
      dxB=(p3x-xA);
      duB=(p3u-uA);
      dvB=(p3v-vA);
      dzB=(p3z-zA);
    }

    byte *b=dest+p1y*SCRWID;
    int  *z=wb_zbuf+(p1y*SCRWID*2)+wb_offset;

    do{
      int xAs=xA>>8;
      int xBs=xB>>8;

      if(xBs>xAs)      wb_drawTxtZ(xAs,xBs,uA,vA,zA,uB,vB,zB,b+xAs,z+xAs);
      else if(xBs<xAs) wb_drawTxtZ(xBs,xAs,uB,vB,zB,uA,vA,zA,b+xBs,z+xBs);

      dyA--;
      dyB--;
      if(dyA<0&&dyB<0) break;

      xA+=dxA;
      uA+=duA;
      vA+=dvA;
      zA+=dzA;

      xB+=dxB;
      uB+=duB;
      vB+=dvB;
      zB+=dzB;

      if(!dyA){
        dyA=p3y-p2y;
        if(dyA){
          dxA=(p3x-p2x)/dyA;
          duA=(p3u-p2u)/dyA;
          dvA=(p3v-p2v)/dyA;
          dzA=(p3z-p2z)/dyA;
        }else{
          dxA=(p3x-p2x);
          duA=(p3u-p2u);
          dvA=(p3v-p2v);
          dzA=(p3z-p2z);
        }
      }else if(!dyB){
        dyB=p2y-p3y;
        if(dyB){
          dxB=(p2x-p3x)/dyB;
          duB=(p2u-p3u)/dyB;
          dvB=(p2v-p3v)/dyB;
          dzB=(p2z-p3z)/dyB;
        }else{
          dxB=(p2x-p3x);
          duB=(p2u-p3u);
          dvB=(p2v-p3v);
          dzB=(p2z-p3z);
        }
      }
      b+=SCRWID;
      z+=SCRWID*2;
    }while(dyA>=0&&dyB>=0);
    df++;
  }while(--i);
  df-=count;
}

void wb_init()
{
   O=new obj;
   readobj(O,"data\\blob.x");
   df=new int[O->faces];    
   loadmap("data\\wormhole",wb_texture,wb_pal);
}

void wb_precalc()
{
   for(float v=0;v<2*PI;v+=2*PI/2500){
      float vv=v;
      if(kbhit()) return;
      for( float z=2; z<400;z+=0.4 )
      {
         unsigned int x=cos(vv)*z+320;
         unsigned int y=sin(vv)*z+200;

         if(x<640&&y<400)
         {
            wb_hole[x+y*640]=(int((v*384)/PI)&0xff)+((int(10000/z)&0xff)<<8);
            wb_zbuf[x+y*640]=(30000-(1000*z));
         }

         // v*384/PI = v*768 / 2*PI  (3 repeats per circle, txtmap 256 wide)
         // 6000/z                   (indicates repeats along z)
      }
   frameCount=0;
   }
}

void wb_deinit()
{
   delete[] O->v;
   delete[] O->f;
   delete O;
}

void wb_drawhole()
{
   static int fy=0;
   fy += 768;
   int *b=(int*)buf;
   short *h=wb_hole+wb_offset;
   int y=200; 
   do{
      int x=40;
      do{
        int tmp=wb_texture[ (fy+*h++)&0xffff ];
        tmp   +=wb_texture[ (fy+*h++)&0xffff ]<<8;
        tmp   +=wb_texture[ (fy+*h++)&0xffff ]<<16;    
        tmp   +=wb_texture[ (fy+*h++)&0xffff ]<<24;
        *b++=tmp;
        tmp    =wb_texture[ (fy+*h++)&0xffff ];
        tmp   +=wb_texture[ (fy+*h++)&0xffff ]<<8;    
        tmp   +=wb_texture[ (fy+*h++)&0xffff ]<<16;    
        tmp   +=wb_texture[ (fy+*h++)&0xffff ]<<24;
        *b++=tmp;
      }while(--x);
      h+=320;
   }while(--y);
}  
